package com.olympya.tools.grok;

import java.util.*;
import java.util.concurrent.*;

/**
 * A generic <em>software artifact</em> (e.g., a java source file; a bug report; etc).
 *
 * @see Repository
 *
 * @author Paulo Mattos
 */
public abstract class Artifact
{
  private final String name;
  private final Repository repository;

  /**
   * Constructor designed for subclassing.
   */
  protected Artifact(String name, Repository repository)
  {
    if (name.length() == 0)
      throw new IllegalArgumentException("empty name");

    if (repository == null)
      throw new NullPointerException("repository");

    this.name       = name;
    this.repository = repository;
  }

  /**
   * Returns artifact symbolic name.
   */
  public final String getName()
  {
    return name;
  }

  /**
   * Returns artifact latest revision.
   */
  public abstract Revision latestRevision();

  /**
   * Returns an immutable list containg all artifact's revisions in <em>descending</em> order.
   */
  public abstract List<Artifact.Revision> allRevisions();

  /**
   * Returns this artifact repository.
   */
  public final Repository getRepository()
  {
    return repository;
  }

  /**
   * Returns a short textual description of this artifact.
   */
  @Override
  public String toString()
  {
    return String.format("%s[%s]", getArtifactClass().getSimpleName(), getName());
  }

  private Class<? extends Artifact> getArtifactClass()
  {
    return getRepository().getType().getArtifactClass();
  }

  /**
   * An artifact revision.
   */
  public static abstract class Revision
  {
    private final Artifact artifact;
    private final Repository.Modification modification;

    /**
     * Constructor designed for subclassing.
     */
    protected Revision(Artifact artifact, Repository.Modification modification)
    {
      if (artifact == null)
        throw new NullPointerException("artifact");

      if (modification == null)
        throw new NullPointerException("modification");

      this.artifact     = artifact;
      this.modification = modification;
    }

    /**
     * Returns the artifact described by this revision.
     */
    public final Artifact getArtifact()
    {
      return artifact;
    }

    /**
     * Returns the repository modification containing this revision.
     */
    public final Repository.Modification getModification()
    {
      return modification;
    }

    /**
     *
     * @throws ClassCastException
     *
     * @see #size()
     */
    public abstract Future<Integer> countChanges(Artifact.Revision base);

    /**
     *
     * @see #countChanges(Artifact.Revision)
     */
    public abstract Future<Integer> size();

    /**
     * Returns a short textual description of this artifact's revision.
     */
    @Override
    public String toString()
    {
      return String.format("%s@%s", getArtifact(), getModification().toNumber());
    }
  }
}
