package com.olympya.tools.grok;

import java.util.*;
import java.util.regex.*;
import javax.xml.bind.annotation.*;
import javax.xml.bind.annotation.adapters.*;

import static com.google.common.collect.Maps.*;

import com.olympya.util.xml.*;
import com.olympya.util.collect.*;
import com.olympya.util.lang.*;

/**
 * A software developer. Each instance represents a unique registered developer.
 * <p>
 *
 * Instances of this class support full {@linkplain javax.xml.bind JAXB}-based
 * XML serialization and deserialization.
 *
 * @author Paulo Mattos
 */
public final class Developer
implements Comparable<Developer>
{
  /**
   * Regular expression for validating email syntax.
   */
  public static final Pattern EMAIL_REGEX = Pattern.compile("[a-zA-Z0-9.]+@[a-z]+\\.[a-z]+");

  @XmlAttribute(required=true)
  private final String name;

  @XmlAttribute
  private final String email;

  @XmlAttribute
  private final String im;

  @XmlElement(name="user-names")
  @XmlJavaTypeAdapter(XmlUserNamesAdapter.class)
  @Immutable
  private final Map<Repository.Type, String> userNames;

  private Developer(String name, String email, String im, Map<Repository.Type, String> userNames)
  {
    assert name      != null;
    assert userNames != null;

    this.name      = name;
    this.email     = email;
    this.im        = im;
    this.userNames = userNames;
  }

  /**
   * No-arg constructor required by the {@link javax.xml.bind} API.
   */
  @SuppressWarnings("UnusedDeclaration")
  private Developer()
  {
    name      = null;
    email     = null;
    im        = null;
    userNames = newEnumMap(Repository.Type.class);
  }

  /**
   * Returns developer's real name (e.g., {@code "Frodo} {@code Baggins"}).
   */
  public String getName()
  {
    return name;
  }

  /**
   *
   */
  public String getEmail()
  {
    return email;
  }

  /**
   * Returns developer <em>instant messaging</em> nickname.
   */
  public String getIM()
  {
    return im;
  }

  /**
   *
   */
  public boolean hasUserName(Repository.Type repoType)
  {
    if (repoType == null)
      throw new NullPointerException("repoType");

    return userNames.containsKey(repoType);
  }

  /**
   *
   * @throws IllegalArgumentException
   */
  public String getUserName(Repository.Type repoType)
  {
    if (!hasUserName(repoType))
      throw new IllegalArgumentException("repoType=" + repoType);

    return userNames.get(repoType);
  }

  public int compareTo(Developer dev)
  {
    return this.name.compareTo(dev.name);
  }

  /**
   *
   */
  @Override
  public boolean equals(Object obj)
  {
    return obj instanceof Developer && name.equals(((Developer) obj).name);
  }

  /**
   *
   */
  @Override
  public int hashCode()
  {
    return name.hashCode();
  }

  /**
   *
   */
  @Override
  public String toString()
  {
    return name;
  }

  /**
   * Builder of {@link Developer} objects.
   */
  public static class Builder
  {
    private String name;
    private String email;
    private String im;
    private Shared<Map<Repository.Type, String>> userNames = Shared.newEnumMap(Repository.Type.class);

    /**
     * Creates an empty developer builder.
     */
    public Builder()
    {
      name  = null;
      email = null;
      im    = null;
    }

    /**
     * Creates developer builder initialized with the specified {@code Developer} instance.
     */
    public Builder(Developer dev)
    {
      name  = dev.name;
      email = dev.email;
      im    = dev.im;
      userNames.setImmutable(dev.userNames);
    }

    /**
     *
     */
    public String name()
    {
      return name;
    }

    /**
     *
     */
    public Builder name(String name)
    {
      if (name == null)
        throw new NullPointerException("name");

      this.name = name;
      return this;
    }

    /**
     *
     */
    public String email()
    {
      return email;
    }

    /**
     *
     */
    public Builder email(String email)
    {
      if (email != null && !EMAIL_REGEX.matcher(email).matches())
        throw new IllegalArgumentException("email=" + email);

      this.email = email;
      return this;
    }

    /**
     *
     */
    public String im()
    {
      return im;
    }

    /**
     *
     */
    public Builder im(String im)
    {
      this.im = im;
      return this;
    }

    /**
     *
     */
    public Map<Repository.Type, String> userNames()
    {
      return userNames.toImmutable();
    }

    /**
     *
     */
    public Builder userNames(Repository.Type repoType, String userName)
    {
      if (repoType == null)
        throw new NullPointerException("repoType");

      userNames.asMutable().put(repoType, userName);
      return this;
    }

    /**
     *
     */
    public Builder singleUserName(String singleUserName)
    {
      if (singleUserName == null)
        throw new NullPointerException("singleUserName");

      Map<Repository.Type, String> map = newEnumMap(Repository.Type.class);

      for (Repository.Type type : Repository.Type.values())
        map.put(type, singleUserName);

      return this;
    }

    /**
     * Builds a {@code Developer} instance based on this builder current state.
     *
     * @throws IllegalStateException if builder current state isn't capable of producing a valid developer instance.
     */
    public Developer build()
    {
      if (name == null)
        throw new IllegalStateException("name is null");

      return new Developer(name, email, im, userNames.toImmutable());
    }
  }

  private static final class XmlUserNamesAdapter
  extends XmlEnumMapAdapter<Repository.Type, String>
  {
    XmlUserNamesAdapter() {super(Repository.Type.class, true);}

    protected String marshalValue(String value)
    {
      return value;
    }

    protected String unmarshalValue(String value)
    {
      return value;
    }
  }
}
