package com.olympya.tools.grok.intellij;

import java.util.*;
import static java.util.Collections.*;
import javax.swing.*;
import javax.xml.bind.*;
import javax.xml.bind.annotation.*;
import javax.xml.parsers.*;

import org.jdom.*;
import org.jdom.Element;
import org.jdom.output.*;
import org.jdom.input.*;

import org.jetbrains.annotations.*;
import com.intellij.openapi.components.*;
import com.intellij.openapi.options.*;
import com.intellij.openapi.application.*;
import com.intellij.openapi.project.*;
import com.intellij.openapi.vfs.*;

import static com.google.common.collect.Sets.*;

import com.olympya.tools.grok.intellij.ui.*;
import com.olympya.tools.grok.intellij.ui.conf.*;
import com.olympya.tools.grok.metrics.*;

/**
 * Plugin <em>application-level</em> component.
 *
 * <h3>XML Serialization & Deserialization</h3>
 *
 * All serialized Java types are annotated using the annotations defined by the {@link javax.xml.bind.annotation} package.
 * The current implementation also dependes on the following additional APIs: {@link javax.xml.bind}, {@link org.w3c.dom} and {@link org.jdom}.
 * <p>
 *
 * The serialization process includes the following major steps:
 * <ol>
 *   <li> {@linkplain Marshaller Marshalling} the {@link Workgroup} set into a {@link org.w3c.dom.Document org.w3c.dom.Document} object;
 *   <li> Converting the resulting {@link org.w3c.dom.Document org.w3c.dom.Document} to a {@link org.jdom.Document org.jdom.Document} object;
 *   <li> Serializing the root {@link org.jdom.Element} using the {@link PersistentStateComponent} API.
 * </ol>
 *
 * And the deserialization involves the following corresponding steps:
 * <ol>
 *   <li> Wrapping the received root {@link org.jdom.Element} in a {@link org.jdom.Document org.jdom.Document}.
 *   <li> Converting the created {@link org.jdom.Document org.jdom.Document} to a {@link org.w3c.dom.Document org.w3c.dom.Document} object;
 *   <li> {@linkplain Unmarshaller Unmarshalling} the {@link org.w3c.dom.Document org.w3c.dom.Document} to a {@link Workgroup} set object;
 * </ol>
 *
 * <h3>XML Schema Evolution</h3>
 *
 * The...
 *
 * @author Paulo Mattos
 */
@State
(
  name     = Plugin.NAME,
  storages = {@Storage(id="grok", file="$APP_CONFIG$/grok.xml")}
)
public class Plugin
implements ApplicationComponent, Configurable, PersistentStateComponent<Element>
{
  static final String NAME = "Grok";

  private Set<Workgroup> allWorkgroups = emptySortedSet();
  private ConfigurationDialog dialog = null;

  static
  {
    Workgroup.register(new WeightedChangesMetric(3));
    Workgroup.register(new WeightedChangesMetric(7));
    Workgroup.register(new WeightedChangesMetric(15));
    Workgroup.register(new RandomMetric());
  }

  /**
   * Creates plugin object.
   */
  public Plugin()
  {
  }

  /**
   * Returns plugin singleton instance.
   */
  public static Plugin getInstance()
  {
    Application app = ApplicationManager.getApplication();
    Plugin singleton = app.getComponent(Plugin.class);
    assert singleton != null;

    return singleton;
  }

  /**
   * Returns an immutable set containing all currently defined workgroups.
   */
  @NotNull
  public Set<Workgroup> getWorkgroups()
  {
    return allWorkgroups;
  }

  /**
   * Returns workgroup instance fo the specified versioned project file.
   *
   * @throws NoSuchWorkgroupException
   */
  public Workgroup getWorkgroupFor(Project project, VirtualFile path) throws NoSuchWorkgroupException
  {
    return Workgroup.of(allWorkgroups, project, path);
  }

  /**
   * Replaces all currently defined workgroups by the specified collection.
   */
  public void setWorkgroups(@NotNull Collection<? extends Workgroup> wgroups)
  {
    if (wgroups == null)
      throw new NullPointerException("wgroups");

    this.allWorkgroups = newHashSet();
    this.allWorkgroups.addAll(wgroups);
    this.allWorkgroups = unmodifiableSet(this.allWorkgroups);
  }

  /**
   * Returns the configuration dialog.
   */
  public ConfigurationDialog getConfigurationDialog()
  {
    if (dialog == null)
      dialog = new ConfigurationDialog();

    return dialog;
  }

  //
  // ApplicationComponent API
  //

  /**
   * {@inheritDoc}
   */
  public void initComponent()
  {
  }

  /**
   * {@inheritDoc}
   */
  public void disposeComponent()
  {
  }

  /**
   * {@inheritDoc}
   */
  @NotNull
  public String getComponentName()
  {
    return NAME;
  }

  //
  // Configurable API
  //

  /**
   * {@inheritDoc}
   */
  @Nls
  public String getDisplayName()
  {
    return "Grokkers";
  }

  /**
   * {@inheritDoc}
   */
  @Nullable
  public Icon getIcon()
  {
    return Icons.GROK_LOGO;
  }

  /**
   * {@inheritDoc}
   */
  @Nullable
  @NonNls
  public String getHelpTopic()
  {
    return null;
  }

  /**
   * {@inheritDoc}
   */
  public JComponent createComponent()
  {
    return getConfigurationDialog().getRootComponent();
  }

  /**
   * {@inheritDoc}
   */
  public boolean isModified()
  {
    return dialog != null && dialog.isModified();
  }

  /**
   * {@inheritDoc}
   */
  public void apply() throws ConfigurationException
  {
    if (dialog != null)
      allWorkgroups = dialog.getWorkgroups();
  }

  /**
   * {@inheritDoc}
   */
  public void reset()
  {
    if (dialog != null)
      dialog.setWorkgroups(allWorkgroups);
  }

  /**
   * {@inheritDoc}
   */
  public void disposeUIResources()
  {
    dialog = null;
  }

  //
  // PersistentStateComponent API
  //

  /**
   * Returns the current plugin state as a JDOM element.
   * <p>
   *
   * See the {@link Plugin} documentation for details.
   */
  public org.jdom.Element getState()
  {
    try
    {
      DocumentBuilderFactory wdocBuilder = DocumentBuilderFactory.newInstance();
      org.w3c.dom.Document wdoc = wdocBuilder.newDocumentBuilder().newDocument();

      JAXBContext context = JAXBContext.newInstance(PersistentState.class);
      context.createMarshaller().marshal(new PersistentState(this), wdoc);

      return new DOMBuilder().build(wdoc).getRootElement();
    }
    catch (Exception e)
    {
      e.printStackTrace();
      assert false : e;
    }

    return null;
  }

  /**
   * Loads the plugin new state from a JDOM element.
   * <p>
   *
   * See the {@link Plugin} documentation for details.
   */
  public void loadState(org.jdom.Element jroot)
  {
    PersistentState.checkSchemaVersion(jroot);
    try
    {
      org.jdom.Document jdoc = new Document().setRootElement(jroot);

      org.w3c.dom.Document wdoc  = new DOMOutputter().output(jdoc);
      org.w3c.dom.Element  wroot = wdoc.getDocumentElement();

      JAXBContext context = JAXBContext.newInstance(PersistentState.class);
      PersistentState loadedState = (PersistentState) context.createUnmarshaller().unmarshal(wroot);

      loadedState.load(this);
    }
    catch (JDOMException e)
    {
      e.printStackTrace();
      assert false;
    }
    catch (JAXBException e)
    {
      e.printStackTrace();
      assert false;
    }
  }

  /**
   * The Intellij storage {@code <component>} XML element (i.e., the plugin XML tree root element).
   */
  @XmlRootElement(name="component")
  private static final class PersistentState
  {
    private static final int    SCHEMA_VERSION = 1;
    private static final String SCHEMA_VERSION_ATTRIB = "schema-version";

    @XmlAttribute(name=SCHEMA_VERSION_ATTRIB, required=true)
    @SuppressWarnings("UnusedDeclaration")
    private final int schemaVersion = SCHEMA_VERSION;

    @XmlElementWrapper(name="all-workgroups")
    @XmlElement(name="workgroup")
    @Nullable
    private final Set<Workgroup> allWorkgroups;

    /**
     * Saves plugin state <em>before</em> serialization.
     */
    PersistentState(Plugin component)
    {
      assert component != null && component.allWorkgroups != null;
      this.allWorkgroups = component.allWorkgroups;
    }

    /**
     * No-arg constructor required by the {@link javax.xml.bind} API.
     */
    @SuppressWarnings("UnusedDeclaration")
    private PersistentState()
    {
      allWorkgroups = null;
    }

    /**
     * Loads plugin state <em>after</em> deserialization.
     */
    void load(Plugin component)
    {
      if (allWorkgroups != null)
        component.allWorkgroups = unmodifiableSet(allWorkgroups);
      else
        component.allWorkgroups = emptySortedSet();
    }

    static void checkSchemaVersion(org.jdom.Element jroot)
    {
      Attribute attrib = jroot.getAttribute(SCHEMA_VERSION_ATTRIB);
      assert attrib != null;

      try
      {
        assert attrib.getIntValue() == SCHEMA_VERSION;
      }
      catch (DataConversionException e)
      {
        assert false : e;
      }
    }
  }
}