package com.olympya.tools.grok.intellij;

import java.net.*;
import java.util.*;
import java.util.concurrent.*;
import java.lang.ref.*;
import static java.util.Collections.*;

import com.intellij.openapi.project.*;
import com.intellij.openapi.vcs.*;
import com.intellij.openapi.vcs.history.*;
import com.intellij.openapi.vfs.*;
import com.intellij.vcsUtil.*;
import org.incava.util.diff.*;
import static com.google.common.collect.Lists.*;

import com.olympya.tools.grok.*;
import static com.olympya.tools.grok.Repository.Type.*;

/**
 * A {@link AbstractVcs}-based implementation of {@code VersionControl}.
 *
 * @see Source
 *
 * @author Paulo Mattos
 */
public final class VersionControlMapper
extends VersionControl
{
  private final Workgroup workgroup;
  private final WeakReference<Project> project;
  private final Map<VirtualFile, SourceMapper> cachedSources;
  private final Executor executor = Executors.newCachedThreadPool();
//  private final Executor executor = Executors.newSingleThreadExecutor();

  /**
   * Returns {@link VersionControl} for the specified project file.
   */
  VersionControlMapper(Workgroup workgroup, Project project)
  {
    assert workgroup != null && project != null;

    this.workgroup     = workgroup;
    this.project       = new WeakReference<Project>(project);
    this.cachedSources = new WeakHashMap<VirtualFile, SourceMapper>();
  }

  /**
   * Returns repository root URI.
   */
  public URI getRoot()
  {
    return workgroup.getRepository(Type.VERSION_CONTROL).getRoot();
  }

  /**
   * Maps the specified {@code VirtualFile} to the corresponding {@code Source} artifact.
   */
  public Source asSource(VirtualFile file) throws NoSuchArtifactException
  {
    if (cachedSources.containsKey(file))
      return cachedSources.get(file);

    FilePath filePath = VcsUtil.getFilePath(file.getPath());
    AbstractVcs vcs = ProjectLevelVcsManager.getInstance(project.get()).getVcsFor(filePath);

    if (vcs == null || !vcs.fileExistsInVcs(filePath))
      throw new NoSuchArtifactException(getType(), filePath.getName());

    if (!containsFilePath(vcs, file))
      throw new NoSuchArtifactException(getType(), filePath.getName());

    SourceMapper src = new SourceMapper(filePath);
    cachedSources.put(file, src);

    return src;
  }

  /**
   * Unmaps the specified {@code Source} artifact to the corresponding {@code VirtualFile}.
   *
   * @throws IllegalArgumentException
   */
  public VirtualFile asVirtualFile(Source src)
  {
    if (!(src instanceof SourceMapper))
      throw new IllegalArgumentException("src=" + src);

    return ((SourceMapper) src).file.getVirtualFile();
  }

  private boolean containsFilePath(AbstractVcs vcs, VirtualFile file) throws NoSuchArtifactException
  {
    URI fileURI = getFileURI(vcs, file);
    return !getRoot().relativize(fileURI).isAbsolute();
  }

  /**
   * Current implementation models each revision as a unique modification (i.e., ignores composite change list).
   */
  private Modification asModification(VcsFileRevision fileRev)
  {
    Developer author = workgroup.getDeveloper(getType(), fileRev.getAuthor());
    Date date = fileRev.getRevisionDate();

    long id;
    VcsRevisionNumber rev = fileRev.getRevisionNumber();

    if (rev instanceof VcsRevisionNumber.Int)
      id = ((VcsRevisionNumber.Int) rev).getValue();
    else if (rev instanceof VcsRevisionNumber.Long)
      id = ((VcsRevisionNumber.Long) rev).getLongValue();
    else
      id = Long.parseLong(rev.asString()); // ...use string as a fallback

    return numericModification(author, date, id);
  }

  //
  // Helper methods
  //

  static AbstractVcs getVcsFor(Project project, VirtualFile file) throws NoSuchArtifactException
  {
    ProjectLevelVcsManager vcsManager = ProjectLevelVcsManager.getInstance(project);

    FilePath filePath = VcsUtil.getFilePath(file.getPath());
    AbstractVcs vcs = vcsManager.getVcsFor(filePath);

    if (vcs == null)
      throw new NoSuchArtifactException(VERSION_CONTROL, filePath.getName());

    return vcs;
  }

  static URI getRepositoryURI(Project project, VirtualFile file) throws NoSuchArtifactException
  {
    AbstractVcs vcs = VersionControlMapper.getVcsFor(project, file);

    VirtualFile vroot = VcsUtil.getVcsRootFor(project, file);
    return getFileURI(vcs, vroot);
  }

  static URI getFileURI(AbstractVcs vcs, VirtualFile file)
  {
    FilePath filePath = VcsUtil.getFilePath(file.getPath());
    RepositoryLocation fileLoc = vcs.getCommittedChangesProvider().getLocationFor(filePath);
    assert fileLoc != null;

    try
    {
      return new URI(fileLoc.toString());
    }
    catch (URISyntaxException e)
    {
      assert false : e;
      return null;
    }
  }

  private static List<VcsFileRevision> sortRevisions(List<VcsFileRevision> revisions)
  {
    Comparator<VcsFileRevision> cmp = new Comparator<VcsFileRevision>()
    {
      public int compare(VcsFileRevision r1, VcsFileRevision r2)
      {
        return r1.getRevisionNumber().compareTo(r2.getRevisionNumber());
      }
    };

    List<VcsFileRevision> sortedRevions = newArrayList(revisions);
    Collections.sort(sortedRevions, Collections.reverseOrder(cmp));

    return sortedRevions;
  }

  /**
   *
   */
  private final class SourceMapper
  extends Source
  {
    private final FilePath file;
    private List<Revision> allRevisions = null;
    private Date allRevisionsNextUpdate = null;

    private SourceMapper(FilePath file)
    {
      super(file.getName(), VersionControlMapper.this);

      this.file = file;
    }

    /**
     * {@inheritDoc}
     */
    public Revision latestRevision()
    {
      return allRevisions().get(0);
    }

    /**
     * {@inheritDoc}
     */
    public List<Revision> allRevisions()
    {
      Date today = new Date();

      if (allRevisions != null && today.before(allRevisionsNextUpdate))
        return allRevisions;

      allRevisions = newArrayList();
      List<VcsFileRevision> revisions = this.getHistory().getRevisionList();

      for (VcsFileRevision fileRev : sortRevisions(revisions))
        allRevisions.add(new RevisionMapper(this, fileRev));

      allRevisions = unmodifiableList(allRevisions);
      allRevisionsNextUpdate = new Date(today.getTime() + 1 * 60 * 60 * 1000);

      return allRevisions;
    }

    private VcsHistorySession getHistory()
    {
      AbstractVcs vcs = ProjectLevelVcsManager.getInstance(project.get()).getVcsFor(file);
      assert vcs != null;

      try
      {
        return vcs.getVcsHistoryProvider().createSessionFor(file);
      }
      catch (VcsException e)
      {
        assert false : e;
      }

      return null;
    }
  }

  /**
   *
   */
  private final class RevisionMapper
  extends Source.Revision
  {
    private final VcsFileRevision fileRev;
    private String[] cachedContent = null;

    private RevisionMapper(SourceMapper src, VcsFileRevision fileRev)
    {
      super(src, asModification(fileRev));
      this.fileRev = fileRev;
    }

    /**
     * {@inheritDoc}
     */
    public Future<Integer> countChanges(final Artifact.Revision base)
    {
      if (!(base instanceof RevisionMapper))
        throw new IllegalArgumentException("base=" + base);

      Callable<Integer> count = new Callable<Integer>()
      {
        @SuppressWarnings("unchecked")
        public Integer call()
        {
          String[] baseContent = ((RevisionMapper) base).getContent();
          String[] thisContent = getContent();

          List<Difference> diffs = (List<Difference>) new Diff(baseContent, thisContent).diff();

          int count = 0;
          for (Difference diff : diffs)
          {
            if (diff.getAddedEnd() != -1)
              count += diff.getAddedEnd() - diff.getAddedStart();

            if (diff.getDeletedEnd() != -1)
              count += diff.getDeletedEnd() - diff.getDeletedStart();
          }
          return count;
        }
      };

      FutureTask<Integer> futureCount = new FutureTask<Integer>(count);
      executor.execute(futureCount);
      return futureCount;
    }

    /**
     * {@inheritDoc}
     */
    public Future<Integer> size()
    {
      Callable<Integer> size = new Callable<Integer>()
      {
        public Integer call()
        {
          return getContent().length + 1;
        }
      };

      FutureTask<Integer> futureSize = new FutureTask<Integer>(size);
      executor.execute(futureSize);
      return futureSize;
    }

    private synchronized String[] getContent()
    {
      if (cachedContent != null)
        return cachedContent;

      try
      {
        fileRev.loadContent();
        String content = new String(fileRev.getContent());

        return cachedContent = content.split("\n");
      }
      catch (Exception e)
      {
        assert false : e;
        return null;
      }
    }
  }
}
