package com.olympya.tools.grok.intellij;

import java.net.*;
import java.util.*;
import java.lang.reflect.*;
import javax.xml.bind.annotation.*;
import javax.xml.bind.annotation.adapters.*;

import com.intellij.openapi.project.*;
import com.intellij.openapi.vfs.*;
import com.intellij.openapi.vcs.*;
import static com.google.common.collect.Sets.*;
import static com.google.common.collect.Lists.*;
import static com.google.common.collect.Maps.newEnumMap;

import com.olympya.util.xml.*;
import com.olympya.util.collect.*;
import com.olympya.util.lang.*;
import com.olympya.tools.grok.*;
import com.olympya.tools.grok.metrics.*;
import static com.olympya.tools.grok.Repository.Type.*;

/**
 * A development workgroup.
 * <p>
 *
 * Instances of this class support full {@linkplain javax.xml.bind JAXB}-based
 * XML serialization and deserialization.
 *
 * @see Team
 *
 * @author Paulo Mattos
 */
public final class Workgroup
implements Comparable<Workgroup>
{
  private static List<Metric> defaultMetrics = newArrayList(); // is good to have stable ordering

  @XmlAttribute(required=true)
  private final String name;

  @XmlElement(name="repositories")
  @XmlJavaTypeAdapter(XmlRepositoriesAdapter.class)
  @Immutable
  private final Map<Repository.Type, RepositoryInfo> repositories;

  @XmlElement(name="metrics")
  @XmlJavaTypeAdapter(XmlMetricsAdapter.class)
  @Immutable
  private final Map<Repository.Type, Metric> metrics;

  @XmlElementWrapper(name="developers")
  @XmlElement(name="member")
  @Immutable
  private final SortedSet<Developer> developers;

  private final Map<Project, VersionControlMapper> vcsCache;

  {
    vcsCache = new WeakHashMap<Project, VersionControlMapper>();
  }

  /**
   * No-arg constructor required by the {@link javax.xml.bind} API.
   *
   * @todo Needs review
   */
  @SuppressWarnings("UnusedDeclaration")
  private Workgroup()
  {
    this.name         = null;
    this.repositories = newEnumMap(Repository.Type.class);
    this.metrics      = newEnumMap(Repository.Type.class);
    this.developers   = newTreeSet();
  }

  private Workgroup(String name, Map<Repository.Type, RepositoryInfo> repositories,
                    Map<Repository.Type, Metric> metrics, SortedSet<Developer> developers)
  {
    assert name         != null;
    assert repositories != null;
    assert metrics      != null;
    assert developers   != null;

    this.name         = name;
    this.repositories = repositories;
    this.metrics      = metrics;
    this.developers   = developers;
  }

  /**
   * Returns workgroup symbolic name.
   */
  public String getName()
  {
    return name;
  }

  /**
   *
   * @param type
   */
  public RepositoryInfo getRepository(Repository.Type type)
  {
    return repositories.get(type);
  }

  /**
   *
   */
  public VersionControlMapper getVersionControl(Project project)
  {
    if (vcsCache.containsKey(project))
      return vcsCache.get(project);

    VersionControlMapper vc = new VersionControlMapper(this, project);
    vcsCache.put(project, vc);
    return vc;
  }

  /**
   *
   */
  public Metric getMetric(Repository.Type repoType)
  {
    if (repoType == null)
      throw new NullPointerException("repoType");

    return metrics.get(repoType);
  }

  /**
   *
   */
  public SortedSet<Developer> getDevelopers()
  {
    return developers;
  }

  /**
   *
   */
  Developer getDeveloper(Repository.Type repoType, String userName)
  {
    for (Developer developer : developers)
      if (developer.hasUserName(repoType) && developer.getUserName(repoType).equals(userName))
        return developer;

    return new Developer.Builder().name(userName).build();

//    throw new IllegalArgumentException("userName=" + userName);
  }

  /**
   *
   */
  public Team asTeam(Project project)
  {
    if (project == null)
      throw new NullPointerException("project");

    return new Team(developers, getVersionControl(project));
  }

  /**
   * {@inheritDoc}
   */
  public int compareTo(Workgroup wgroup)
  {
    return this.name.compareTo(wgroup.name);
  }

  /**
   * Returns a short textual description of this workgroup.
   */
  public String toString()
  {
    return String.format("%s[%s, %s, %s]", getClass().getSimpleName(), name, developers.size(), repositories.get(VERSION_CONTROL));
  }

  //
  // Utilities Methods
  //

  /**
   *
   * @throws NoSuchWorkgroupException
   */
  static Workgroup of(Collection<? extends Workgroup> wgroups, URI repoRoot)
                      throws NoSuchWorkgroupException
  {
    for (Workgroup wgroup : wgroups)
      if (wgroup.getRepository(Repository.Type.VERSION_CONTROL).getRoot().equals(repoRoot))
        return wgroup;

    throw new NoSuchWorkgroupException("No workgroup defined for " + repoRoot, null);
  }

  /**
   * Returns workgroup instance fo the specified versioned project file.
   *
   * @throws NoSuchWorkgroupException
   */
  static Workgroup of(Collection<? extends Workgroup> wgroups, Project project,
                      VirtualFile path) throws NoSuchWorkgroupException
  {
    URI vcsURI;
    AbstractVcs vcs;
    try
    {
      vcs = VersionControlMapper.getVcsFor(project, path);
      vcsURI = VersionControlMapper.getRepositoryURI(project, path);
    }
    catch (NoSuchArtifactException e)
    {
      throw new NoSuchWorkgroupException(e);
    }

    for (Workgroup wgroup : wgroups)
      if (isRelative(wgroup.getRepository(Repository.Type.VERSION_CONTROL).getRoot(), vcsURI))
        return wgroup;

    String major = String.format("No workgroup defined for '%s'", path.getName());
    String minor = VersionControlMapper.getFileURI(vcs, path).toString();
    throw new NoSuchWorkgroupException(major, minor);
  }

  private static boolean isRelative(URI repo, URI root)
  {
    return !repo.relativize(root).isAbsolute();
  }

  /**
   * Registers the specified default metric.
   *
   * @see #defaultMetrics()
   */
  public static void register(Metric defaultMetric)
  {
    if (defaultMetric == null)
      throw new NullPointerException("defaultMetric");

    defaultMetrics.add(defaultMetric);
  }

  /**
   * Returns all known metrics (created with sensible default parameters).
   *
   * @see #register(Metric)
   */
  public static List<Metric> defaultMetrics()
  {
    return immutableList(defaultMetrics);
  }

  /**
   * Builder of {@link Workgroup} objects.
   */
  public static class Builder
  {
    private String name;
    private Shared<Map<Repository.Type, RepositoryInfo>> repositories = Shared.newEnumMap(Repository.Type.class);
    private Shared<Map<Repository.Type, Metric>> metrics = Shared.newEnumMap(Repository.Type.class);
    private Shared<SortedSet<Developer>> developers = Shared.newTreeSet();

    /**
     * Creates an empty workgroup builder.
     */
    public Builder()
    {
      name = null;
    }

    /**
     * Creates workgroup builder initialized with the specified {@code Workgroup} instance.
     */
    public Builder(Workgroup wgroup)
    {
      name = wgroup.name;
      repositories.setImmutable(wgroup.repositories);
      metrics.setImmutable(wgroup.metrics);
      developers.setImmutable(wgroup.developers);
    }

    /**
     *
     */
    public String name()
    {
      return name;
    }

    /**
     *
     */
    public Builder name(String name)
    {
      if (name == null)
        throw new NullPointerException("name");

      this.name = name;
      return this;
    }

    /**
     *
     */
    public Map<Repository.Type, RepositoryInfo> repositories()
    {
      return repositories.toImmutable();
    }

    /**
     *
     *
     * @throws IllegalArgumentException
     */
    public Builder repository(Repository.Type type, RepositoryInfo info)
    {
      if (type == null)
        throw new NullPointerException("type");

      repositories.asMutable().put(type, info);
      return this;
    }

    /**
     *
     */
    public Map<Repository.Type, Metric> metrics()
    {
      return metrics.toImmutable();
    }

    /**
     *
     */
    public Builder metric(Repository.Type repoType, Metric metric)
    {
      if (repoType == null)
        throw new NullPointerException("repoType");

      metrics.asMutable().put(repoType, metric);
      return this;
    }

    /**
     *
     */
    public SortedSet<Developer> developers()
    {
      return developers.toImmutable();
    }

    /**
     *
     */
    public Builder developer(Developer dev)
    {
      if (dev == null)
        throw new NullPointerException("dev");

      developers.asMutable().add(dev);
      return this;
    }

    /**
     *
     */
    public Builder clearDevelopers()
    {
      developers.asMutable().clear();
      return this;
    }

    /**
     * Builds a {@code Workgroup} instance based on this builder current state.
     *
     * @throws IllegalStateException if builder current state isn't capable of producing a valid workgroup instance.
     */
    public Workgroup build()
    {
      if (name == null)
        throw new IllegalStateException("name is null");

      if (repositories.asMutable().get(Repository.Type.VERSION_CONTROL) == null)
        throw new IllegalStateException("version control repository is null");

      return new Workgroup(name, repositories.toImmutable(), metrics.toImmutable(), developers.toImmutable());
    }
  }

  private static final class XmlMetricsAdapter
  extends XmlEnumMapAdapter<Repository.Type, Metric>
  {
    XmlMetricsAdapter() {super(Repository.Type.class, true);}

    protected String marshalValue(Metric value)
    {
      if (value == null) return "";

      String params = "";

      // @todo temporary hack
      if (value instanceof WeightedChangesMetric)
        params += "@" + Integer.toString(((WeightedChangesMetric) value).getElapsedDays());

      return value.getClass().getName() + params;
    }

    protected Metric unmarshalValue(String metricClassName)
    {
      if (metricClassName.isEmpty()) return null;

      try
      {
        if (metricClassName.startsWith(WeightedChangesMetric.class.getName()))
        {
          String[] nameAndParam = metricClassName.split("@");

          Class<?> clazz = Class.forName(nameAndParam[0]);
          Constructor ctr = clazz.getConstructor(Integer.TYPE);
          return (Metric) ctr.newInstance(Integer.parseInt(nameAndParam[1]));
        }
        else
        {
          Class<?> clazz = Class.forName(metricClassName);
          return (Metric) clazz.newInstance();
        }
      }
      catch (Exception e)
      {
        assert false : e;
        return null;
      }
    }
  }

  private static final class XmlRepositoriesAdapter
  extends XmlEnumMapAdapter<Repository.Type, RepositoryInfo>
  {
    XmlRepositoriesAdapter() {super(Repository.Type.class, true);}

    protected String marshalValue(RepositoryInfo value)
    {
      return value.toString();
    }

    protected RepositoryInfo unmarshalValue(String riString)
    {
      return RepositoryInfo.fromString(riString);
    }
  }
}
