package com.olympya.tools.grok.intellij.ui;

import java.net.*;
import javax.swing.*;

import com.intellij.ide.*;
import com.intellij.openapi.vfs.*;
import jetbrains.communicator.core.*;
import jetbrains.communicator.core.users.*;
import jetbrains.communicator.core.vfs.*;
import jetbrains.communicator.ide.*;
import jetbrains.communicator.idea.*;
import jetbrains.communicator.util.*;

import com.olympya.tools.grok.*;
import com.olympya.tools.grok.intellij.*;

/**
 * A team communication action.
 *
 * @author Hugo Baés
 */
public enum Send
{
  /**
   * E-mail action.
   */
  EMAIL("Send e-mail", Icons.EMAIL)
  {
    public void run(Developer target, Query query)
    {
      String filename = query.artifact.getArtifact().getName();
      String filepath = "/no/path/to/file/.../yet"; //TODO: context.artifact.getLocation();
      String revision = String.valueOf(query.artifact.getModification().toNumber());
      String date     = query.artifact.getModification().getDate().toString();

      String subject = "I need your help on the following artifact: "+filename;
      String body    = "Path: " + filepath +" \n" +
                       "Revision: " + revision + "\n" +
                       "Date: " + date + "\n" +
                       "\n";

      try
      {
        URI url = new URI("mailto", target.getEmail() + "?subject=" + subject + "&body=" + body, null);
        BrowserUtil.launchBrowser(url.toASCIIString());
      }
      catch (URISyntaxException e)
      {
        BrowserUtil.launchBrowser("mailto:" + target.getEmail() + "?subject=" + subject);
      }
    }

    public boolean isDeveloperAvailable(Developer target, Query query)
    {
      return target.getEmail() != null && target.getEmail().length() > 0;
    }
  },

  /**
   * Instante message action (through IDEtalk).
   */
  INSTANT_MESSAGE("Start IDETalk", Icons.IDETALK)
  {
    private User user;
    private EventBroadcaster broadcaster;

    public void run(Developer target, Query query)
    {
      VirtualFile virtual = query.context.vfile;
      VFile file          = VFSUtil.createFileFrom(virtual, query.context.project);

      IDEFacade ideFacade = (IDEFacade) Pico.getInstance().getComponentInstanceOfType(IDEFacade.class);
      String comment      = ideFacade.getMessage(StringUtil.getMsg("code_pointer.message.input"), StringUtil.getMsg("code_pointer.message.input.title"), null);

      CodePointer pointer;
      if (query.context.startLine > -1)
      {
        if (query.context.endLine > -1)
          pointer = new CodePointer(query.context.startLine, query.context.startColumn, query.context.endLine, query.context.endColumn);
        else
          pointer = new CodePointer(query.context.startLine, query.context.startColumn);
      }
      else
        pointer = new CodePointer(1, 1);

      user.sendCodeIntervalPointer(file, pointer, comment, broadcaster);
    }

    public boolean isDeveloperAvailable(Developer target, Query query)
    {
      return false;

/* @bug Show Stopper

      UserModel userModel = (UserModel) Pico.getInstance().getComponentInstanceOfType(UserModel.class);
      broadcaster = userModel.getBroadcaster();

      User[] users = userModel.getAllUsers();
      for (int i = 0; i < users.length; i++)
      {
        if (target.getIM().equals(users[i].getDisplayName()))
        {
          user = users[i];
          return true;
        }
      }
      return false;
*/
    }
  },

  /**
   * Bug report action.
   */
  BUG_REPORT("Submit a bug report", Icons.BUG_REPORT)
  {
    public void run(Developer target, Query query)
    {
      String trackerName = query.workgroup.getRepository(Repository.Type.ISSUE_TRACKER).getName();
      AbstractIssueTracker tracker = AbstractIssueTracker.getIssueTrackerFromName(trackerName);
      assert tracker != null;
      tracker.submitBugReport(target, query);
    }
    public boolean isDeveloperAvailable(Developer target, Query query)
    {
      return query.workgroup.getRepository(Repository.Type.ISSUE_TRACKER) != null;
    }
  },

  /**
   * Feature request action.
   */
  FEATURE_REQUEST("Submit a feature request", Icons.FEATUER_REQUEST)
  {
    public void run(Developer target, Query query)
    {
      String trackerName = query.workgroup.getRepository(Repository.Type.ISSUE_TRACKER).getName();
      AbstractIssueTracker tracker = AbstractIssueTracker.getIssueTrackerFromName(trackerName);
      assert tracker != null;
      tracker.submitFeatureRequest(target, query);
    }
    public boolean isDeveloperAvailable(Developer target, Query query)
    {
      return query.workgroup.getRepository(Repository.Type.ISSUE_TRACKER) != null;
    }
  };

  private final String displayName;
  private final Icon icon;

  private Send(String displayName, Icon icon)
  {
    assert displayName != null;
    assert icon != null;
    this.displayName = displayName;
    this.icon = icon;
  }

  /**
   * Execute action for the specified developer, in response to an event.
   */
  public abstract void run(Developer target, Query query);

  /**
   * Check if the action can be done to the specified developer.
   */
  public abstract boolean isDeveloperAvailable(Developer target, Query query);

  /**
   * Returns the icon of the action.
   */
  public Icon getIcon()
  {
    return icon;
  }

  /**
   * Returns the string to be displayed in the action popup menu.
   */
  public String getDisplayName()
  {
    return displayName;
  }
}
