package com.olympya.tools.grok.intellij.ui;

import com.intellij.ui.LayeredIcon;

import javax.swing.*;
import javax.swing.border.Border;
import java.awt.*;
import java.awt.color.ColorSpace;
import java.awt.event.MouseEvent;
import java.awt.image.*;

/**
 * An IntelliJ-toolbar-like button.
 *
 * @author Hugo Baés
 */
public class ToolbarButton
   extends JButton
{
  private boolean inside;

  private Border outBorder;
  private Color  outBg;

  private Border overBorder;
  private Color  overBg;

  private Border pressedBorder;
  private Color  pressedBg;

  /**
   * Creates an toolbar button with the specified icon.
   */
  public ToolbarButton(Icon icon)
  {
    super(icon);

    if (icon == null)
      throw new NullPointerException("icon");

    Dimension size = new Dimension(25, 25);
    outBorder      = BorderFactory.createEmptyBorder();
    outBg          = getBackground();
    overBorder     = BorderFactory.createLineBorder(new Color(8, 36, 107));
    overBg         = new Color(181, 190, 214);
    pressedBorder  = overBorder;
    pressedBg      = new Color(130, 146, 185);

    setOpaque(false);
    setContentAreaFilled(false);
    setBorder(outBorder);
    setFocusable(false);
    setMinimumSize(size);
    setPreferredSize(size);
    setMaximumSize(size);

    setDisabledIcon(createDisabledIcon(icon, true));
    setPressedIcon(createPressedIcon(icon));

  }

  /**
   * Creates an icon in grayscale.
   * @param idelike if the new icon will be like IntelliJ toolbar ou natural
   */
  public static Icon createDisabledIcon(final Icon icon, boolean idelike)
  {
    // Create blank image buffer
    final BufferedImage image = new BufferedImage(icon.getIconWidth(), icon.getIconHeight(), BufferedImage.TYPE_INT_ARGB);
    final Graphics2D graphics = image.createGraphics();

    // Paint in a fake component using image graphics
    graphics.setColor(new Color(0, 0, 0, 0)); // Transparent
    graphics.fillRect(0, 0, icon.getIconWidth(), icon.getIconHeight());
    icon.paintIcon(new JLabel(), graphics, 0, 0); // "canvas" from a dummy component
    graphics.dispose();

    // Apply image filter
    if (idelike)
    {
      return new ImageIcon(GrayFilter.createDisabledImage(image));
    }
    else
    {
      ColorSpace colorSpace = ColorSpace.getInstance(ColorSpace.CS_GRAY);
      ColorConvertOp toGray = new ColorConvertOp(colorSpace, null);
      BufferedImage grayImage = toGray.filter(image, null);
      return new ImageIcon(grayImage);
    }
  }

  /**
   * Creates an icon traslated 1 pixel to right and down.
   */
  public static Icon createPressedIcon(Icon icon)
  {
    Icon blankIcon = new ImageIcon();
    LayeredIcon pressed = new LayeredIcon(2);
    pressed.setIcon(blankIcon, 0);
    pressed.setIcon(icon, 1, 1, 1);
    return pressed;
  }

  /**
   * Sets the border of this button when mouse is out of it.
   */
  public void setMouseOutBorder(Border border)
  {
    outBorder = border;
  }

  /**
   * Returns the border of this button when mouse is out of it.
   */
  public Border getMouseOutBorder()
  {
    return outBorder != null ? outBorder : getBorder();
  }

  /**
   * Sets the background of this button when mouse is out of it.
   */
  public void setMouseOutBackground(Color bg)
  {
    outBg = bg;
  }

  /**
   * Returns the background of this button when mouse is out of it.
   */
  public Color getMouseOutBackground()
  {
    return outBg != null ? outBg : getBackground();
  }

  /**
   * Sets the border of this button when mouse is over it.
   */
  public void setMouseOverBorder(Border border)
  {
    overBorder = border;
  }

  /**
   * Returns the border of this button when mouse is over it.
   */
  public Border getMouseOverBorder()
  {
    return overBorder != null ? overBorder : getMouseOutBorder();
  }

  /**
   * Sets the background of this button when mouse is over it.
   */
  public void setMouseOverBackground(Color bg)
  {
    overBg = bg;
  }

  /**
   * Returns the background of this button when mouse is over it.
   */
  public Color getMouseOverBackground()
  {
    return overBg != null ? overBg : getMouseOutBackground();
  }

  /**
   * Sets the border of this button when mouse button is pressed.
   */
  public void setMousePressedBorder(Border border)
  {
    pressedBorder = border;
  }

  /**
   * Returns the border of this button when mouse button is pressed.
   */
  public Border getMousePressedBorder()
  {
    return pressedBorder != null ? pressedBorder : getMouseOverBorder();
  }

  /**
   * Sets the background of this button when mouse button is pressed.
   */
  public void setMousePressedBackground(Color bg)
  {
    pressedBg = bg;
  }

  /**
   * Returns the background of this button when mouse button is pressed.
   */
  public Color getMousePressedBackground()
  {
    return pressedBg!=null ? pressedBg : getMouseOverBackground();
  }

  @Override
  protected void processMouseEvent(MouseEvent e)
  {
    super.processMouseEvent(e);

    switch (e.getID())
    {
      case MouseEvent.MOUSE_ENTERED:
        inside = true;
        break;
      case MouseEvent.MOUSE_EXITED:
        inside = false;
        break;
    }

    boolean leftPressed = (e.getModifiers() & MouseEvent.BUTTON1_DOWN_MASK) != 0;

    if (inside && isEnabled())
    {
      setOpaque(true);
      setContentAreaFilled(true);
      if (leftPressed)
      {
        super.setBorder(getMousePressedBorder());
        super.setBackground(getMousePressedBackground());
      }
      else
      {
        super.setBorder(getMouseOverBorder());
        super.setBackground(getMouseOverBackground());
      }
    }
    else
    {
      setOpaque(false);
      setContentAreaFilled(false);
      super.setBorder(getMouseOutBorder());
      super.setBackground(getMouseOutBackground());
    }
  }
}