package com.olympya.tools.grok.intellij.ui;

import com.intellij.openapi.actionSystem.*;
import com.intellij.openapi.editor.*;
import com.intellij.openapi.project.*;
import com.intellij.openapi.vfs.*;
import com.intellij.psi.*;
import org.jetbrains.annotations.*;

/**
 * Stores UI-centric objects.
 *
 * @author Hugo Baés
 */
public final class UIContext
{
  /**
   * Current open project.
   */
  @NotNull
  public final Project project;

  /**
   * Virtual file of the context.
   */
  public final VirtualFile vfile;

  /**
   * Psi file of the context.
   */
  public final PsiFile psiFile;

  /**
   * Psi element of the context.
   */
  public final PsiElement psiElement;

  /**
   * Editor of the context.
   */
  @Nullable
  public final Editor editor;

  /**
   * Cursor line position or selection start line.
   * <code>-1</code> if editor is not open.
   */
  public final int startLine;

  /**
   * Cursor column position or selection start column.
   * <code>-1</code> if editor is not open.
   */
  public final int startColumn;

  /**
   * Selection end line.
   * <code>-1</code> if editor is not open or if there is no selection.
   */
  public final int endLine;

  /**
   * Selection end column.
   * <code>-1</code> if editor is not open or if there is no selection.
   */
  public final int endColumn;

  /**
   * Stores context objects.
   */
  public UIContext(DataContext context)
  {
    project     = DataKeys.PROJECT     .getData(context);
    vfile       = DataKeys.VIRTUAL_FILE.getData(context);
    editor      = DataKeys.EDITOR      .getData(context);
    psiFile     = DataKeys.PSI_FILE    .getData(context);
    psiElement  = DataKeys.PSI_ELEMENT .getData(context);

    if (editor != null)
    {
      int selectionStart = editor.getSelectionModel().getSelectionStart();
      int selectionEnd   = editor.getSelectionModel().getSelectionEnd();

      if (selectionStart != selectionEnd)
      {
        LogicalPosition start = editor.offsetToLogicalPosition(selectionStart);
        LogicalPosition end   = editor.offsetToLogicalPosition(selectionEnd);
        startLine   = start.line;
        startColumn = start.column;
        endLine     = end.line;
        endColumn   = end.column;
      }
      else
      {
        LogicalPosition pos = editor.getCaretModel().getLogicalPosition();
        startLine   = pos.line;
        startColumn = pos.column;
        endLine     = -1;
        endColumn   = -1;
      }
    }
    else
    {
      startLine   = -1;
      startColumn = -1;
      endLine     = -1;
      endColumn   = -1;
    }
  }
}
