package com.olympya.tools.grok.intellij.ui.conf;

import java.util.*;
import javax.swing.*;
import javax.swing.event.*;
import javax.swing.tree.*;

import static com.google.common.collect.Sets.*;

import com.olympya.tools.grok.intellij.*;

/**
 * Tree model backed by a {@link Workgroup} collection.
 *
 * @author Hugo Baés
 * @author Paulo Mattos
 */
class AllWorkgroupsTreeModel
implements TreeModel
{
  private final JTree view;
  private final SortedSet<WorkgroupTreeNode> nodes = newTreeSet();
  private final EventListenerList listeners = new EventListenerList();
  private boolean modified;

  /**
   * Create an empty tree.
   */
  public AllWorkgroupsTreeModel(JTree view)
  {
    this(view, null);
  }

  /**
   * Create a tree based on a set of workgroup.
   */
  public AllWorkgroupsTreeModel(JTree view, Collection<? extends Workgroup> wgroups)
  {
    if (view == null)
      throw new NullPointerException("view");

    this.view = view;
    this.view.setModel(this);

    if (wgroups != null)
      for (Workgroup workgroup : wgroups)
        nodes.add(new WorkgroupTreeNode(this, workgroup));

    modified = false;
    valueForPathChanged(getRootPath(), nodes);
  }

  /**
   * Returns true if tree was modified.
   */
  public boolean isModified()
  {
    return modified;
  }

  /**
   * Notifies model of any changes.
   */
  public void modified()
  {
    modified = true;
  }

  /**
   * Add a new workgroup to the tree.
   */
  public boolean addWorkgroup()
  {
    WorkgroupTreeNode workgroupNode = new WorkgroupTreeNode(this);

    String name = workgroupNode.getName();
    int i = 2;

    while (!nodes.add(workgroupNode))
      workgroupNode.setName(name + " " + String.valueOf(i++));

    modified();
    valueForPathChanged(getRootPath(), workgroupNode);

    TreePath path = getRootPath().pathByAddingChild(workgroupNode);
    view.scrollPathToVisible(path);
    view.setSelectionPath(path);

    return true;
  }

  /**
   * Removes the specified workgroup from the tree.
   */
  public boolean removeWorkgroup(WorkgroupTreeNode workgroupNode)
  {
    if (nodes.remove(workgroupNode))
    {
      workgroupNode.setParentModel(null);

      modified();
      valueForPathChanged(getRootPath(), null);
      return true;
    }
    return false;
  }


  /**
   * Creates a sorted set of workgroups based on the tree information.
   */
  public SortedSet<Workgroup> toWorkgroups(boolean resetModified)
  {
    SortedSet<Workgroup> workgroups = newTreeSet();

    for (WorkgroupTreeNode node : nodes)
      workgroups.add(node.toWorkgroup());

    if (resetModified)
      modified = false;

    return workgroups;
  }

  /**
   * Returns the workgroup tree view.
   */
  public JTree getView()
  {
    return view;
  }

  /**
   * Returns the path to the root.
   */
  public TreePath getRootPath()
  {
    return new TreePath(nodes);
  }

  /**
   * {@inheritDoc}
   */
  public SortedSet<WorkgroupTreeNode> getRoot()
  {
    return nodes;
  }

  /**
   * {@inheritDoc}
   */
  public Object getChild(Object parent, int index)
  {
    if (parent instanceof DeveloperTreeNode)
      return null;

    if (parent instanceof WorkgroupTreeNode)
      return ((WorkgroupTreeNode) parent).getChildAt(index);

    if (parent == nodes)
      return nodes.toArray()[index];

    throw new ClassCastException("Invalid node type");
  }

  /**
   * {@inheritDoc}
   */
  public int getChildCount(Object parent)
  {
    if (parent instanceof DeveloperTreeNode)
      return 0;

    if (parent instanceof WorkgroupTreeNode)
      return ((WorkgroupTreeNode) parent).getChildCount();

    if (parent == nodes)
      return nodes.size();

    throw new ClassCastException("Invalid node type");
  }

  /**
   * {@inheritDoc}
   */
  public boolean isLeaf(Object node)
  {
    if (node instanceof DeveloperTreeNode)
      return true;

    if (node instanceof WorkgroupTreeNode)
      return false;

    if (node == nodes)
      return false;

    throw new ClassCastException("Invalid node type");
  }

  /**
   * {@inheritDoc}
   * <p>
   *
   * The last component of path is the same newValue if it just changed
   * its presentation and/or children, otherwise the last component is it's parent.
   * If newValue is null, it's was removed.
   */
  public void valueForPathChanged(TreePath path, Object newValue)
  {
    System.out.println("Tree Changed!");
    System.out.println("Old path: "+path);
    System.out.println("Old value: "+path.getLastPathComponent());
    System.out.println("New value: "+newValue);

    for (TreeModelListener l : listeners.getListeners(TreeModelListener.class))
      l.treeStructureChanged(new TreeModelEvent(this, path));

    if (path.getLastPathComponent() != newValue && newValue != null)
    {
      view.scrollPathToVisible(path.pathByAddingChild(newValue));
      view.setSelectionPath(path.pathByAddingChild(newValue));
    }
    else
    {
      view.scrollPathToVisible(path);
      view.setSelectionPath(path);
    }
  }

  /**
   * {@inheritDoc}
   */
  @SuppressWarnings("SuspiciousMethodCalls")
  public int getIndexOfChild(Object parent, Object child)
  {
    if (parent instanceof DeveloperTreeNode)
      return -1;

    if (parent instanceof WorkgroupTreeNode)
      ((WorkgroupTreeNode) parent).getIndex((DeveloperTreeNode) child);

    if (parent == nodes && nodes.contains(child)) //returns false if child type is not compatible
    {
      int i = 0;

      for (WorkgroupTreeNode node : nodes)
      {
        if (node == child) return i;
        i++;
      }
      return -1;
    }

    return -1;
  }

  /**
   * {@inheritDoc}
   */
  public void addTreeModelListener(TreeModelListener l)
  {
    listeners.add(TreeModelListener.class, l);
  }

  /**
   * {@inheritDoc}
   */
  public void removeTreeModelListener(TreeModelListener l)
  {
    listeners.remove(TreeModelListener.class, l);
  }
}
