package com.olympya.tools.grok.intellij.ui.conf;

import java.awt.*;
import java.awt.event.*;
import java.util.*;
import javax.swing.*;
import javax.swing.event.*;
import javax.swing.tree.*;

import com.intellij.uiDesigner.core.*;
import static com.google.common.collect.Sets.newTreeSet;

import com.olympya.tools.grok.*;
import com.olympya.tools.grok.intellij.*;
import static com.olympya.tools.grok.intellij.ui.UIs.*;
import com.olympya.tools.grok.intellij.ui.*;

/**
 * Grok configuration dialog.
 *
 * @author Hugo Baés
 */
public class ConfigurationDialog
{
  private JTree wgroupsView;
  private JButton add;
  private JButton rem;
  private JPanel rootPanel;
  private JLabel leftTopCorner;
  private JLabel rightTopCorner;
  private JLabel centerTopText;
  private JPanel infoPanel;

  private final SortedSet<Workgroup> wgroups = newTreeSet();
  private AllWorkgroupsTreeModel wgroupsModel;

  /**
   * Creates an instance of the Configuration Form and loads the workgroup info.
   */
  public ConfigurationDialog()
  {
    $$$setupUI$$$();

    add.addActionListener(new ActionListener()
    {
      public void actionPerformed(ActionEvent e)
      {
        createConfigAddPopup(ConfigurationDialog.this).showUnderneathOf(add);
      }
    });

    rem.addActionListener(new ActionListener()
    {
      public void actionPerformed(ActionEvent e)
      {
        performRemoveAction();
      }
    });
  }

  /**
   * Returns the main panel of the component configuration.
   */
  public JComponent getRootComponent()
  {
    return rootPanel;
  }

  /**
   * Returns the current workgroup information from configuration dialog.
   */
  public Set<Workgroup> getWorkgroups()
  {
    try
    {
      return wgroupsModel.toWorkgroups(true);
    }
    catch (Exception e)
    {
      // TODO: catch errors
      e.printStackTrace();
      assert false;
      return null;
    }
  }

  /**
   * Refreshes configuration dialog data with current workgroups information.
   */
  public void setWorkgroups(Collection<? extends Workgroup> wgroups)
  {
    this.wgroups.clear();
    this.wgroups.addAll(wgroups);

    wgroupsModel = new AllWorkgroupsTreeModel(wgroupsView, this.wgroups);
  }

  /**
   * Returns true if any data was modified.
   */
  public boolean isModified()
  {
    return wgroupsModel.isModified();
  }

  /**
   * Returns the selected workgroup in the tree or null
   * if selection path doesn't include any workgroup.
   */
  public WorkgroupTreeNode getSelectedWorkgroup()
  {
    TreePath path = wgroupsView.getSelectionPath();

    if (path == null)
      return null;

    if (path.getLastPathComponent() instanceof WorkgroupTreeNode)
      return (WorkgroupTreeNode) path.getLastPathComponent();

    if (path.getLastPathComponent() instanceof DeveloperTreeNode)
      return (WorkgroupTreeNode) path.getParentPath().getLastPathComponent();

    return null;
  }

  /**
   * Returns the selected developer in the tree or {@code null} if selection path
   * doesn't include any developer.
   */
  public DeveloperTreeNode getSelectedDeveloper()
  {
    if (wgroupsView.getLastSelectedPathComponent() instanceof DeveloperTreeNode)
      return (DeveloperTreeNode) wgroupsView.getLastSelectedPathComponent();

    return null;
  }

  /**
   * Performs the "Add New Workgroup" action. Returns {@code true} if the action was performed.
   */
  public boolean performAddWorkgroupAction()
  {
    return wgroupsModel.addWorkgroup();
  }

  /**
   * Performs the "Add New Developer" action. Returns {@code true} if the action was performed.
   */
  public boolean performAddDeveloperAction()
  {
    WorkgroupTreeNode workgroup = getSelectedWorkgroup();

    return workgroup != null && workgroup.addDeveloper();
  }

  /**
   * Performs the "Edit Developer" action, called by the action popup menu.
   * Returns {@code true} if the action was performed.
   */
  public void performEditDeveloperAction(Workgroup workgroup, Developer developer)
  {
    if (workgroup == null)
      throw new NullPointerException("workgroup");

    if (developer == null)
      throw new NullPointerException("developer");

    // get workgroup position
    int i = 0;
    for (Workgroup w : wgroups)
    {
      if (w.equals(workgroup))
        break;
      i++;
    }
    assert i < wgroups.size();

    Object root = wgroupsModel.getRoot();
    WorkgroupTreeNode parent = (WorkgroupTreeNode) wgroupsModel.getChild(root, i);
    TreePath path = new TreePath(new Object[]{root, parent});

    // get developer position (if workgroup contains it)
    int j = 0;
    for (Developer d : workgroup.getDevelopers())
    {
      if (d.equals(developer))
        break;
      j++;
    }

    if (j < workgroup.getDevelopers().size())
    {
      wgroupsView.setSelectionPath(path.pathByAddingChild(parent.getChildAt(j)));
    }
    else
    {
      parent.addDeveloper(new DeveloperTreeNode(parent));
    }
  }

  private boolean performRemoveAction()
  {
    DeveloperTreeNode developer = getSelectedDeveloper();
    WorkgroupTreeNode workgroup = getSelectedWorkgroup();

    if (developer != null)
      return performRemoveDeveloperAction(developer);

    else if (workgroup != null)
      return performRemoveWorkgroupAction(workgroup);

    return false;
  }

  private boolean performRemoveDeveloperAction(DeveloperTreeNode developer)
  {
    assert developer != null;
    assert developer.getParent() != null;

    int option = JOptionPane.showConfirmDialog(rootPanel,
                                               "Do you want to remove " + developer.getName() + " developer?",
                                               "Remove developer?",
                                               JOptionPane.YES_NO_OPTION,
                                               JOptionPane.QUESTION_MESSAGE);

    if (option == JOptionPane.YES_OPTION)
    {
      WorkgroupTreeNode parent = developer.getParent();
      return parent.removeDeveloper(developer);
    }

    return false;
  }

  private boolean performRemoveWorkgroupAction(WorkgroupTreeNode workgroupNode)
  {
    assert workgroupNode != null;

    int option = JOptionPane.showConfirmDialog(rootPanel,
                                               "Do you want to remove " + workgroupNode.getName() + " workgroup and all developers on it?",
                                               "Remove workgroup AND its developers?",
                                               JOptionPane.YES_NO_OPTION,
                                               JOptionPane.QUESTION_MESSAGE);

    if (option == JOptionPane.YES_OPTION)
      return wgroupsModel.removeWorkgroup(workgroupNode);

    return false;
  }

  private void showDeveloperInfo(DeveloperTreeNode developer)
  {
    DeveloperPanel developerPanel = new DeveloperPanel(developer);

    centerTopText.setText("Developer Profile");

    infoPanel.removeAll();
    infoPanel.setLayout(new GridLayoutManager(1, 1, new Insets(3, 3, 3, 3), -1, -1));
    infoPanel.add(developerPanel, new GridConstraints(0, 0, 1, 1, GridConstraints.ANCHOR_CENTER, GridConstraints.FILL_BOTH, GridConstraints.SIZEPOLICY_CAN_SHRINK | GridConstraints.SIZEPOLICY_CAN_GROW, GridConstraints.SIZEPOLICY_CAN_SHRINK | GridConstraints.SIZEPOLICY_CAN_GROW, null, null, null, 0, false));
    infoPanel.validate();
    infoPanel.repaint();
  }

  private void showWorkgroupInfo(WorkgroupTreeNode workgroup)
  {
    WorkgroupPanel workgroupPanel = new WorkgroupPanel(workgroup);

    centerTopText.setText("Workgroup Profile");

    infoPanel.removeAll();
    infoPanel.setLayout(new GridLayoutManager(1, 1, new Insets(3, 3, 3, 3), -1, -1));
    infoPanel.add(workgroupPanel, new GridConstraints(0, 0, 1, 1, GridConstraints.ANCHOR_CENTER, GridConstraints.FILL_BOTH, GridConstraints.SIZEPOLICY_CAN_SHRINK | GridConstraints.SIZEPOLICY_CAN_GROW, GridConstraints.SIZEPOLICY_CAN_SHRINK | GridConstraints.SIZEPOLICY_CAN_GROW, null, null, null, 0, false));
    infoPanel.validate();
    infoPanel.repaint();
  }

  private void createUIComponents()
  {
    add = new ToolbarButton(Icons.ADD);
    rem = new ToolbarButton(Icons.EXCLUDE);

    wgroupsView = createWorkgroupTree();

    // TODO: review colors
    Color linecolor = /*UIManager.getColor("MenuItem.borderColor");         //*/ new Color(24, 74, 124);
    Color backcolor = /*UIManager.getColor("MenuItem.selectionBackground"); //*/ new Color(166, 206, 246);
    Color textcolor = /*UIManager.getColor("MenuItem.selectionForeground"); //*/ new Color(0, 0, 0);
    int arcDg = 5;

    leftTopCorner = createTopLeftCorner(linecolor, backcolor, arcDg);
    rightTopCorner = createTopRightCorner(linecolor, backcolor, arcDg);
    centerTopText = createTopLabel(linecolor, backcolor, textcolor);

    infoPanel = new JPanel();
    infoPanel.setBorder(BorderFactory.createLineBorder(linecolor));
  }

  private JTree createWorkgroupTree()
  {
    JTree tree = new JTree();

    // Set model
    wgroupsModel = new AllWorkgroupsTreeModel(tree);

    // Set listener
    tree.getSelectionModel().setSelectionMode(TreeSelectionModel.SINGLE_TREE_SELECTION);
    tree.addTreeSelectionListener(new TreeListener());

    // Set renderer
    DefaultTreeCellRenderer renderer = new DefaultTreeCellRenderer();
    renderer.setOpenIcon(Icons.WORKGROUP);
    renderer.setClosedIcon(Icons.WORKGROUP);
    renderer.setLeafIcon(Icons.DEVELOPER);
    tree.setCellRenderer(renderer);

    // Set drag'n'drop handler
    new DragAndDropTreeHandler(tree);

    return tree;
  }

  private static JLabel createTopLeftCorner(final Color line, final Color bg, final int arc)
  {
    return new JLabel()
    {
      public void paintComponent(Graphics g)
      {
        super.paintComponent(g);

        ((Graphics2D) g).setRenderingHint(RenderingHints.KEY_ANTIALIASING, RenderingHints.VALUE_ANTIALIAS_ON);

        // Draws a Rectangle
        g.setColor(bg);
        g.fillRect(0, 0, getWidth(), getHeight());
        g.setColor(line);
        g.drawLine(0, 0, 0, getHeight());
        g.drawLine(0, 0, getWidth(), 0);

        // Clear top left corner
        g.setColor(getBackground());
        g.fillRect(0, 0, arc, arc);

        // Draws an Arc
        g.setColor(bg);
        g.fillArc(0, 0, arc * 2, arc * 2, 90, 90);
        g.setColor(line);
        g.drawArc(0, 0, arc * 2, arc * 2, 90, 90);
      }
    };
  }

  private static JLabel createTopRightCorner(final Color line, final Color bg, final int arc)
  {
    return new JLabel()
    {
      public void paintComponent(Graphics g)
      {
        super.paintComponent(g);

        ((Graphics2D) g).setRenderingHint(RenderingHints.KEY_ANTIALIASING, RenderingHints.VALUE_ANTIALIAS_ON);

        // Draws a Rectangle
        g.setColor(bg);
        g.fillRect(0, 0, getWidth(), getHeight());
        g.setColor(line);
        g.drawLine(0, 0, getWidth(), 0);
        g.drawLine(getWidth() - 1, 0, getWidth() - 1, getHeight());

        // Clear top right corner
        g.setColor(getBackground());
        g.fillRect(getWidth() - arc, 0, getWidth(), arc);

        // Draws an Arc
        g.setColor(bg);
        g.fillArc(getWidth() - arc * 2 - 1, 0, arc * 2, arc * 2, 0, 90);
        g.setColor(line);
        g.drawArc(getWidth() - arc * 2 - 1, 0, arc * 2, arc * 2, 0, 90);
      }
    };
  }

  private static JLabel createTopLabel(final Color line, final Color bg, final Color fg)
  {
    return new JLabel()
    {
      {
        setForeground(fg);
        setBackground(new Color(0, 0, 0, 0)); // Transparent
      }

      public void paintComponent(Graphics g)
      {
        // Draws a Rectangle
        g.setColor(bg);
        g.fillRect(0, 0, getWidth(), getHeight());

        // Draws upper and lower Lines
        g.setColor(line);
        g.drawLine(0, 0, getWidth(), 0);
        g.drawLine(0, getHeight(), getWidth(), getHeight());

        super.paintComponent(g);
      }
    };
  }

  /**
   * Method generated by IntelliJ IDEA GUI Designer
   * >>> IMPORTANT!! <<<
   * DO NOT edit this method OR call it in your code!
   *
   * @noinspection ALL
   */
  private void $$$setupUI$$$()
  {
    createUIComponents();
    rootPanel = new JPanel();
    rootPanel.setLayout(new GridLayoutManager(2, 2, new Insets(5, 5, 5, 5), -1, 0));
    final JScrollPane scrollPane1 = new JScrollPane();
    scrollPane1.setBackground(UIManager.getColor("List.background"));
    rootPanel.add(scrollPane1, new GridConstraints(1, 0, 1, 1, GridConstraints.ANCHOR_CENTER, GridConstraints.FILL_BOTH, GridConstraints.SIZEPOLICY_CAN_SHRINK | GridConstraints.SIZEPOLICY_WANT_GROW, GridConstraints.SIZEPOLICY_CAN_SHRINK | GridConstraints.SIZEPOLICY_WANT_GROW, null, new Dimension(200, 350), null, 0, false));
    scrollPane1.setBorder(BorderFactory.createTitledBorder(BorderFactory.createEtchedBorder(), null));
    wgroupsView.setAutoscrolls(true);
    wgroupsView.setRootVisible(false);
    wgroupsView.setShowsRootHandles(true);
    scrollPane1.setViewportView(wgroupsView);
    rootPanel.add(infoPanel, new GridConstraints(1, 1, 1, 1, GridConstraints.ANCHOR_CENTER, GridConstraints.FILL_BOTH, GridConstraints.SIZEPOLICY_FIXED, GridConstraints.SIZEPOLICY_CAN_SHRINK | GridConstraints.SIZEPOLICY_CAN_GROW, new Dimension(350, 350), null, new Dimension(350, -1), 0, false));
    final JPanel panel1 = new JPanel();
    panel1.setLayout(new GridLayoutManager(1, 3, new Insets(0, 0, 0, 0), 0, 0));
    rootPanel.add(panel1, new GridConstraints(0, 1, 1, 1, GridConstraints.ANCHOR_SOUTH, GridConstraints.FILL_HORIZONTAL, GridConstraints.SIZEPOLICY_CAN_SHRINK | GridConstraints.SIZEPOLICY_CAN_GROW, GridConstraints.SIZEPOLICY_CAN_SHRINK | GridConstraints.SIZEPOLICY_CAN_GROW, new Dimension(350, 25), null, new Dimension(350, 25), 0, false));
    leftTopCorner.setOpaque(true);
    panel1.add(leftTopCorner, new GridConstraints(0, 0, 1, 1, GridConstraints.ANCHOR_EAST, GridConstraints.FILL_VERTICAL, GridConstraints.SIZEPOLICY_FIXED, GridConstraints.SIZEPOLICY_FIXED, new Dimension(10, -1), null, new Dimension(10, -1), 0, false));
    centerTopText.setFont(new Font(centerTopText.getFont().getName(), Font.BOLD, centerTopText.getFont().getSize()));
    centerTopText.setOpaque(true);
    centerTopText.setText("Grok Settings");
    panel1.add(centerTopText, new GridConstraints(0, 1, 1, 1, GridConstraints.ANCHOR_CENTER, GridConstraints.FILL_BOTH, GridConstraints.SIZEPOLICY_CAN_SHRINK | GridConstraints.SIZEPOLICY_CAN_GROW, GridConstraints.SIZEPOLICY_FIXED, null, null, null, 0, false));
    rightTopCorner.setOpaque(true);
    panel1.add(rightTopCorner, new GridConstraints(0, 2, 1, 1, GridConstraints.ANCHOR_WEST, GridConstraints.FILL_VERTICAL, GridConstraints.SIZEPOLICY_FIXED, GridConstraints.SIZEPOLICY_FIXED, new Dimension(10, -1), null, new Dimension(10, -1), 0, false));
    final JPanel panel2 = new JPanel();
    panel2.setLayout(new GridLayoutManager(1, 3, new Insets(0, 0, 0, 0), 0, 0));
    rootPanel.add(panel2, new GridConstraints(0, 0, 1, 1, GridConstraints.ANCHOR_SOUTH, GridConstraints.FILL_HORIZONTAL, GridConstraints.SIZEPOLICY_CAN_SHRINK | GridConstraints.SIZEPOLICY_CAN_GROW, GridConstraints.SIZEPOLICY_CAN_SHRINK | GridConstraints.SIZEPOLICY_CAN_GROW, new Dimension(-1, 25), new Dimension(-1, 25), new Dimension(-1, 25), 0, false));
    add.setBackground(new Color(-986896));
    panel2.add(add, new GridConstraints(0, 0, 1, 1, GridConstraints.ANCHOR_CENTER, GridConstraints.FILL_HORIZONTAL, GridConstraints.SIZEPOLICY_CAN_SHRINK | GridConstraints.SIZEPOLICY_CAN_GROW, GridConstraints.SIZEPOLICY_FIXED, null, null, null, 0, false));
    rem.setEnabled(false);
    panel2.add(rem, new GridConstraints(0, 1, 1, 1, GridConstraints.ANCHOR_CENTER, GridConstraints.FILL_HORIZONTAL, GridConstraints.SIZEPOLICY_CAN_SHRINK | GridConstraints.SIZEPOLICY_CAN_GROW, GridConstraints.SIZEPOLICY_FIXED, null, null, null, 0, false));
    final JLabel label1 = new JLabel();
    label1.setFont(new Font(label1.getFont().getName(), Font.BOLD, label1.getFont().getSize()));
    label1.setText("Workgroups");
    label1.setDisplayedMnemonic('W');
    label1.setDisplayedMnemonicIndex(0);
    panel2.add(label1, new GridConstraints(0, 2, 1, 1, GridConstraints.ANCHOR_EAST, GridConstraints.FILL_NONE, GridConstraints.SIZEPOLICY_WANT_GROW, GridConstraints.SIZEPOLICY_FIXED, null, null, null, 0, false));
    label1.setLabelFor(scrollPane1);
  }

  /**
   * @noinspection ALL
   */
  public JComponent $$$getRootComponent$$$()
  {
    return rootPanel;
  }

  private class TreeListener
          implements TreeSelectionListener
  {
    public void valueChanged(TreeSelectionEvent e)
    {
      Object node = wgroupsView.getLastSelectedPathComponent();

      System.out.println("Selection changed!");

      if (node instanceof DeveloperTreeNode)
      {
        rem.setEnabled(true);
        showDeveloperInfo((DeveloperTreeNode) node);
      }
      else if (node instanceof WorkgroupTreeNode)
      {
        rem.setEnabled(true);
        showWorkgroupInfo((WorkgroupTreeNode) node);
      }
      else
      {
        rem.setEnabled(false);
        centerTopText.setText("Grok Settings");
        infoPanel.removeAll();
        infoPanel.validate();
        infoPanel.repaint();
      }
    }
  }

}