package com.olympya.tools.grok.intellij.ui.conf;

import java.awt.dnd.*;
import java.awt.*;
import java.awt.datatransfer.*;
import java.awt.image.*;
import javax.swing.*;
import javax.swing.tree.*;

import com.olympya.tools.grok.*;

/**
 * A handler of drag'n'drop operation.
 *
 * @author Hugo Baés
 */
class DragAndDropTreeHandler
        implements DragGestureListener, DragSourceListener, DropTargetListener
{

  private JTree tree;
  private DragSource dragSource;
  private DeveloperTreeNode draggedNode;
  private WorkgroupTreeNode draggedNodeParent;
  private BufferedImage image = null;
  private Rectangle rect2D = new Rectangle();
  boolean drawImage = true;

  public DragAndDropTreeHandler(JTree tree)
  {
    if (tree == null)
      throw new NullPointerException("tree");

    int action = DnDConstants.ACTION_MOVE; //DnDConstants.ACTION_COPY_OR_MOVE;

    this.tree = tree;
    dragSource = new DragSource();
    dragSource.createDefaultDragGestureRecognizer(tree, action, this);
    new DropTarget(tree, action, this);
  }

  public void dragGestureRecognized(DragGestureEvent dge)
  {
    System.out.println("Drag gesture recongnized");
    TreePath path = tree.getSelectionPath();

    if (path != null && path.getLastPathComponent() instanceof DeveloperTreeNode)
    {
      System.out.println("Developer dragged");
      draggedNode = (DeveloperTreeNode) path.getLastPathComponent();
      draggedNodeParent = draggedNode.getParent();

      if (drawImage)
      {
        Rectangle pathBounds = tree.getPathBounds(path);
        JComponent lbl = (JComponent) tree.getCellRenderer().getTreeCellRendererComponent(tree, draggedNode, false, tree.isExpanded(path), tree.getModel().isLeaf(path.getLastPathComponent()), 0, false);
        lbl.setBounds(pathBounds);
        image = new BufferedImage(lbl.getWidth(), lbl.getHeight(), java.awt.image.BufferedImage.TYPE_INT_ARGB_PRE);
        Graphics2D graphics = image.createGraphics();
        graphics.setComposite(AlphaComposite.getInstance(AlphaComposite.SRC_OVER, 0.5f));
        //lbl.setOpaque(false);
        lbl.paint(graphics);
        graphics.dispose();
      }

      dragSource.startDrag(dge, DragSource.DefaultMoveNoDrop, image, new Point(0, 0), new TransferableDeveloperTreeNode(draggedNode), this);
    }
  }

  public void dragEnter(DragSourceDragEvent dsde)
  {
    dragCursors(dsde);
  }

  public void dragOver(DragSourceDragEvent dsde)
  {
    dragCursors(dsde);
  }

  public void dropActionChanged(DragSourceDragEvent dsde)
  {
    dragCursors(dsde);
  }

  public void dragCursors(DragSourceDragEvent dsde)
  {
    switch (dsde.getDropAction())
    {
      case DnDConstants.ACTION_COPY:
        dsde.getDragSourceContext().setCursor(DragSource.DefaultCopyDrop);
        break;

      case DnDConstants.ACTION_MOVE:
        dsde.getDragSourceContext().setCursor(DragSource.DefaultMoveDrop);
        break;

      default:
        dsde.getDragSourceContext().setCursor(DragSource.DefaultMoveNoDrop);
    }
  }

  public void dragExit(DragSourceEvent dse)
  {
    dse.getDragSourceContext().setCursor(DragSource.DefaultMoveNoDrop);
  }

  public void dragDropEnd(DragSourceDropEvent dsde)
  {
    System.out.println("Drag endend...");
    if (dsde.getDropSuccess() && draggedNodeParent != null)
    {
      System.out.print("Drop success: ");
      if (dsde.getDropAction() == DnDConstants.ACTION_MOVE)
      {
        System.out.println("Moved!");
        tree.getModel().valueForPathChanged(draggedNodeParent.getPath(), null);
        tree.getModel().valueForPathChanged(draggedNode.getParent().getPath(), draggedNode);
      }
      if (dsde.getDropAction() == DnDConstants.ACTION_COPY)
      {
        System.out.println("Copied!");
        tree.getModel().valueForPathChanged(draggedNode.getParent().getPath(), draggedNode);
      }
      System.out.println("***");
    }
  }

  public void dragEnter(DropTargetDragEvent dtde)
  {
    dropCursor(dtde);
  }

  public void dragOver(DropTargetDragEvent dtde)
  {
    dropCursor(dtde);
  }

  public void dropActionChanged(DropTargetDragEvent dtde)
  {
    dropCursor(dtde);
  }

  public void dropCursor(DropTargetDragEvent dtde)
  {
    Point pt = dtde.getLocation();
    int action = dtde.getDropAction();
    tree.scrollRectToVisible(new Rectangle(pt, new Dimension(10, 10)));

    if (drawImage)
    {
      paintImage(pt);
    }

    if (canPerformAction(action, pt))
    {
      dtde.acceptDrag(action);
    }
    else
    {
      dtde.rejectDrag();
    }
  }

  public void dragExit(DropTargetEvent dte)
  {
    if (drawImage)
      clearImage();
  }

  public void drop(DropTargetDropEvent dtde)
  {
    try
    {

      if (drawImage)
        clearImage();

      int action = dtde.getDropAction();
      Transferable transferable = dtde.getTransferable();
      Point pt = dtde.getLocation();
      if (transferable.isDataFlavorSupported(TransferableDeveloperTreeNode.NODE_FLAVOR) && canPerformAction(action, pt))
      {
        TreePath pathTarget = tree.getPathForLocation(pt.x, pt.y);
        WorkgroupTreeNode newParentNode = null;

        if (pathTarget.getLastPathComponent() instanceof DeveloperTreeNode)
          newParentNode = ((DeveloperTreeNode) pathTarget.getLastPathComponent()).getParent();

        if (pathTarget.getLastPathComponent() instanceof WorkgroupTreeNode)
          newParentNode = (WorkgroupTreeNode) pathTarget.getLastPathComponent();

        if (executeDrop(newParentNode, action))
        {
          dtde.acceptDrop(action);
          dtde.dropComplete(true);
          return;
        }
      }
      dtde.rejectDrop();
      dtde.dropComplete(false);
    }
    catch (Exception e)
    {
      System.out.println(e);
      dtde.rejectDrop();
      dtde.dropComplete(false);
    }

  }

  public boolean canPerformAction(int action, Point location)
  {
    System.out.print("Can drop? ");
    TreePath pathTarget = tree.getPathForLocation(location.x, location.y);

    if (pathTarget == null)
    {
      System.out.println("No! No path...");
      return false;
    }

    WorkgroupTreeNode workgroup = null;

    if (pathTarget.getLastPathComponent() instanceof DeveloperTreeNode)
      workgroup = ((DeveloperTreeNode) pathTarget.getLastPathComponent()).getParent();

    if (pathTarget.getLastPathComponent() instanceof WorkgroupTreeNode)
      workgroup = (WorkgroupTreeNode) pathTarget.getLastPathComponent();

    if (workgroup != null)
    {
      if (action == DnDConstants.ACTION_COPY)
      {
        tree.setSelectionPath(workgroup.getPath());
        System.out.println("Yes! Copy!");
        return true;
      }
      if (action == DnDConstants.ACTION_MOVE)
      {
        tree.setSelectionPath(workgroup.getPath());
        if  (workgroup != draggedNode.getParent()) // move only to another workgroup
        {
          System.out.println("Yes! Move!");
          return true;
        }
      }
    }

    System.out.println("No!");
    return false;
  }

  public boolean executeDrop(WorkgroupTreeNode newParentNode, int action)
  {
    System.out.print("Execute drop: ");
    if (action == DnDConstants.ACTION_COPY)
    {
      System.out.println("copy!");
      DeveloperTreeNode newDeveloper = new DeveloperTreeNode(newParentNode);

      newDeveloper.setName(draggedNode.getName());
      newDeveloper.setEmail(draggedNode.getEmail());
      newDeveloper.setTalkUsername(draggedNode.getTalkUsername());

      for (Repository.Type type : Repository.Type.values())
      {
        newDeveloper.setUserNames(type, draggedNode.getUserNames(type));
      }

      draggedNode = newDeveloper;
      draggedNodeParent = newDeveloper.getParent();
      return true;
    }

    if (action == DnDConstants.ACTION_MOVE)
    {
      System.out.println("move!");
      newParentNode.addDeveloper(draggedNode);
      return true;
    }
    System.out.println("nothin...");
    return false;
  }

  private final void paintImage(Point pt)
  {
    tree.paintImmediately(rect2D.getBounds());
    rect2D.setRect((int) pt.getX(), (int) pt.getY(), image.getWidth(), image.getHeight());
    tree.getGraphics().drawImage(image, (int) pt.getX(), (int) pt.getY(), tree);
  }

  private final void clearImage()
  {
    tree.paintImmediately(rect2D.getBounds());
  }

}
