package com.olympya.tools.grok.intellij.ui.conf;

import java.net.*;
import java.util.*;
import javax.swing.tree.*;

import com.olympya.tools.grok.*;
import com.olympya.tools.grok.intellij.*;
import com.olympya.tools.grok.metrics.*;

/**
 * A node representing a workgroup.
 *
 * @author Hugo Baés
 */
class WorkgroupTreeNode
        implements Comparable<WorkgroupTreeNode>
{
  private AllWorkgroupsTreeModel model;

  private final Workgroup.Builder builder;
  private final SortedSet<DeveloperTreeNode> developers;

  /**
   * New blank workgroup node.
   */
  public WorkgroupTreeNode(AllWorkgroupsTreeModel parentModel)
  {
    if (parentModel == null)
      throw new NullPointerException("model");

    model      = parentModel;
    developers = new TreeSet<DeveloperTreeNode>();
    builder    = new Workgroup.Builder().name("New Workgroup");
  }

  /**
   * Creates a node representation from the specified developer.
   */
  public WorkgroupTreeNode(AllWorkgroupsTreeModel parentModel, Workgroup workgroup)
  {
    if (parentModel == null)
      throw new NullPointerException("model");

    if (workgroup == null)
      throw new NullPointerException("workgroup");

    model      = parentModel;
    developers = new TreeSet<DeveloperTreeNode>();
    builder    = new Workgroup.Builder(workgroup).clearDevelopers();

    for (Developer d : workgroup.getDevelopers())
    {
      DeveloperTreeNode child = new DeveloperTreeNode(this, d);
      developers.add(child);
    }
  }

  public String getName()
  {
    return builder.name();
  }

  public void setName(String name)
  {
    if (builder.name().equals(name))
      return;

    builder.name(name);
    model.modified();
    model.valueForPathChanged(getPath(), this);
  }

  public String getRepoName(Repository.Type type)
  {
    RepositoryInfo info = builder.repositories().get(type);
    return info != null ? info.getName() : null;
  }

  public void setRepoName(Repository.Type type, String name)
  {
    if (builder.repositories().get(type) != null && builder.repositories().get(type).getName().equals(name))
      return;

    RepositoryInfo info = new RepositoryInfo(name, getRepoRoot(type));
    builder.repository(type, info);
    model.modified();
  }

  public URI getRepoRoot(Repository.Type type)
  {
    RepositoryInfo info = builder.repositories().get(type);
    return info != null ? info.getRoot() : null;
  }

  public void setRepoRoot(Repository.Type type, URI root)
  {
    if (builder.repositories().get(type) != null && builder.repositories().get(type).getRoot().equals(root))
      return;

    RepositoryInfo info = new RepositoryInfo(getRepoName(type), root);
    builder.repository(type, info);
    model.modified();
  }

  public Metric getMetric(Repository.Type type)
  {
    return builder.metrics().get(type);
  }

  public void setMetric(Repository.Type type, Metric metric)
  {
    if (builder.metrics().get(type) != null && builder.metrics().get(type).equals(metric))
      return;

    builder.metric(type, metric);
    model.modified();
  }

  /**
   * Adds a new developer to this workgroup.
   */
  public boolean addDeveloper()
  {
    return addDeveloper(new DeveloperTreeNode(this));
  }

  /**
   * Adds a developer to this workgroup and removes from its
   * old workgroup.
   * Appends an incremetal number if name already exists.
   */
  public boolean addDeveloper(DeveloperTreeNode developerNode)
  {
    if (developerNode.getParent() != this)
    {
      if (developerNode.getParent() != null)
        developerNode.getParent().removeDeveloper(developerNode);

      developerNode.setParent(this);
    }

    String name = developerNode.getName();
    int i = 2;

    while (!developers.add(developerNode))
    {
      developerNode.setName(name+" "+String.valueOf(i++));
    }

    model.modified();
    model.valueForPathChanged(getPath(), developerNode);

    TreePath path = getPath().pathByAddingChild(developerNode);
    model.getView().scrollPathToVisible(path);
    model.getView().setSelectionPath(path);

    return true;
  }

  /**
   * Removes the specified developer from this workgroup.
   * Returns false if workgroup doesn't contain the specified developer.
   */
  public boolean removeDeveloper(DeveloperTreeNode developerNode)
  {
    if (developerNode.getParent() == this && developers.remove(developerNode))
    {
      developerNode.setParent(null);

      model.modified();
      model.valueForPathChanged(getPath(), null);
      return true;
    }

    return false;
  }

  /**
   * Builds a new workgroup from the node information.
   */
  public Workgroup toWorkgroup()
  {
    for (DeveloperTreeNode developer : developers)
      builder.developer(developer.toDeveloper());

    return builder.build();
  }

  /**
   * Returns the path to this workgroup.
   */
  public TreePath getPath()
  {
    if (model != null)
      return new TreePath(new Object[]{model.getRoot(), this});

    return new TreePath(this);
  }

  /**
   * Returns the developer at the specified index.
   */
  public DeveloperTreeNode getChildAt(int childIndex)
  {
    return developers.toArray(new DeveloperTreeNode[developers.size()])[childIndex];
  }

  /**
   * Returns the number of developers in the workgroup.
   */
  public int getChildCount()
  {
    return developers.size();
  }

  /**
   * Used only by its parent {@link AllWorkgroupsTreeModel}.
   */
  void setParentModel(AllWorkgroupsTreeModel parentModel)
  {
    this.model = parentModel;
  }

  /**
   * Returns the model that contains the workgroups.
   */
  public AllWorkgroupsTreeModel getParentModel()
  {
    return model;
  }

  /**
   * Returns the index of the specified developer.
   */
  public int getIndex(DeveloperTreeNode node)
  {
    if (!(node instanceof DeveloperTreeNode))
      return -1;

    int i = 0;
    for (DeveloperTreeNode developer : developers)
    {
      if (developer == node)
        return i;
      i++;
    }
    return -1;
  }

  @Override
  public String toString()
  {
    return getName();
  }

  public int compareTo(WorkgroupTreeNode o)
  {
    return builder.name().compareToIgnoreCase(o.getName());
  }
}
