package com.olympya.tools.grok.metrics;

/**
 * A <em>implementation expertise</em> value (i.e., a quantified, heuristic <em>grokness</em> value).
 * <p>
 *
 * Instances of this class are immutable.
 *
 * @see Metric
 *
 * @author Paulo Mattos
 */
public final class Expertise
implements Comparable<Expertise>
{
  private final float normalizedValue;

  /**
   * Creates an {@code Expertise} instance from the specified normalized value.
   *
   * @throws IllegalArgumentException if {@code normalizedValue} ins't in the range [{@code 0}, {@code 1}].
   */
  public Expertise(float normalizedValue)
  {
    if (normalizedValue < 0 || normalizedValue > 1)
      throw new IllegalArgumentException("normalizedValue=" + normalizedValue);

    this.normalizedValue = normalizedValue;
  }

  /**
   * Creates an {@code Expertise} instance from the specified non-normalized value.
   * The returned expertise will be limited to the range [{@code 0}, {@code 1}].
   */
  public static Expertise clamped(float value)
  {
    return new Expertise(clamp(value, 0, 1));
  }

  private static float clamp(float value, float min, float max)
  {
    return value < min ? min : value > max ? max : value;
  }

  /**
   * Returns a purely random expertise value.
   */
  public static Expertise random()
  {
    return new Expertise((float) Math.random());
  }

  /**
   * Returns this expertise value in the range [{@code 0}, {@code 1}].
   */
  public float normalizedValue()
  {
    return normalizedValue;
  }

  /**
   * Returns this expertise value in the range [{@code 0}, {@code 100}].
   */
  public int percentageValue()
  {
    return Math.round(clamp(normalizedValue*100, 0, 100));
  }

  /**
   * {@inheritDoc}
   */
  public int compareTo(Expertise that)
  {
    return this.percentageValue() - that.percentageValue();
  }

  /**
   * {@inheritDoc}
   */
  @Override
  public boolean equals(Object obj)
  {
    return obj instanceof Expertise && this.percentageValue() == ((Expertise) obj).percentageValue();
  }

  /**
   * {@inheritDoc}
   */
  @Override
  public int hashCode()
  {
    return percentageValue();
  }

  /**
   * Returns a short textual description of this expertise.
   */
  @Override
  public String toString()
  {
    return percentageValue() + "%";
  }
}
