package com.olympya.tools.grok.metrics;

import java.util.*;
import java.util.concurrent.*;

import com.google.common.collect.*;
import static com.google.common.collect.Maps.*;
import static com.google.common.collect.Multimaps.*;

import com.olympya.util.lang.*;
import com.olympya.tools.grok.*;

/**
 * A time-sensitive, weighted metric based on change volume per developer.
 *
 * <img src="WeightedChangesMetric.png"/>
 *
 * @author Paulo Mattos
 */
public final class WeightedChangesMetric
extends Metric
{
  private static final int MIN_REVISIONS = 5;

  private final int elapsedDays;

  /**
   * Creates metric with specified parameters.
   *
   * @throws IllegalArgumentException if {@code days} is non-positive.
   */
  public WeightedChangesMetric(int elapsedDays)
  {
    if (elapsedDays <= 0)
      throw new IllegalArgumentException("days=" + elapsedDays);

    this.elapsedDays = elapsedDays;
  }

  /**
   * Returns elapsed days parameter.
   */
  public int getElapsedDays()
  {
    return elapsedDays;
  }

  /**
   * {@inheritDoc}
   */
  public Map<Developer, Expertise> evaluate(Team team, Artifact artifact)
  {
    System.out.println("==" + getClass().getSimpleName() + "==");

    Multimap<Developer, Callable<Sample>> partialResults = evaluateMeans(artifact);
    Map<Developer, Expertise> results = newHashMap();

    for (Developer author : partialResults.keySet())
    {
      float expertise = 0;
      float weights   = 0;

      for (Callable<Sample> csample : partialResults.get(author))
      {
        try
        {
          Sample s = csample.call();
          expertise += s.value * s.weight;
          weights   += s.weight;
        }
        catch (Exception e)
        {
          assert false : e;
        }
      }
      results.put(author, Expertise.clamped(expertise / weights));
    }

    System.out.println("");

    return results;
  }

  private Multimap<Developer, Callable<Sample>> evaluateMeans(Artifact artifact)
  {
    Multimap<Developer, Callable<Sample>> partialResults = newHashMultimap();

    final Date today = new Date();
    final Date last  = new Date(today.getTime() - elapsedDays * 24 * 60 * 60 * 1000);

    Artifact.Revision r1 = null;
    int revCount = 0;

    for (Artifact.Revision r0 : artifact.allRevisions()) // goes back in time
    {
      Date date0 = r0.getModification().getDate();

      if (date0.before(last) && revCount > MIN_REVISIONS)
        break;

      if (r1 != null)
      {
        System.out.println("rev=" + r1.getModification().toNumber());

        final Artifact.Revision defrev  = r1;
        final Future<Integer>   changes = r1.countChanges(r0);
        final Future<Integer>   size    = r1.size();

        Callable<Sample> computeRatio = new Callable<Sample>()
        {
          public Sample call() throws Exception
          {
            float ratio  = (changes.get() / (float) size.get());
            float weight = cognitiveMemory(today, defrev.getModification().getDate());

            return new Sample(ratio, weight);
          }
        };

        partialResults.put(r1.getModification().getAuthor(), computeRatio);
        revCount++;
      }
      r1 = r0;
    }

    return partialResults;
  }

  /**
   *
   */
  protected float cognitiveMemory(Date today, Date rev)
  {
    assert today.after(rev);

    return 1;
  }

  /**
   * {@inheritDoc}
   */
  public String getDescription()
  {
    return "Evaluates the expertise of a give artifact between revisions ranging from the head to " + elapsedDays + " days ago.";
  }

  /**
   * {@inheritDoc}
   */
  @Override
  public String toString()
  {
    return String.format("%s (last %d days)", getClass().getSimpleName(), elapsedDays);
  }

  /**
   * A single weighted sample for a given developer.
   */
  @Immutable
  private static final class Sample
  {
    final float value, weight;

    Sample(float value, float weight)
    {
      this.value  = value;
      this.weight = weight;
    }
  }
}
