package com.olympya.util.collect;

import java.util.*;

/**
 * An efficient collection building/sharing policy.
 *
 * @author Paulo Mattos
 */
public abstract class Shared<C>
{
  private C collection;
  private boolean isImmutable;

  /**
   * Constructor designed for subclassing.
   */
  protected Shared()
  {
    collection  = create();
    isImmutable = false;
  }

  //
  // Factory Methods
  //

  /**
   * Creates a shared, empty {@link ArrayList} instance.
   */
  public static <E> Shared<List<E>> newArrayList()
  {
    return new Shared<List<E>>()
    {
      protected List<E> create()
      {
        return new ArrayList<E>();
      }

      protected List<E> unmodifiable(List<E> collection)
      {
        return Collections.unmodifiableList(collection);
      }

      protected List<E> copy(List<E> collection)
      {
        return new ArrayList<E>(collection);
      }
    };
  }

  /**
   * Creates a shared, empty {@link HashSet} instance.
   */
  public static <E> Shared<Set<E>> newHashSet()
  {
    return new Shared<Set<E>>()
    {
      protected Set<E> create()
      {
        return new HashSet<E>();
      }

      protected Set<E> unmodifiable(Set<E> collection)
      {
        return Collections.unmodifiableSet(collection);
      }

      protected Set<E> copy(Set<E> collection)
      {
        return new HashSet<E>(collection);
      }
    };
  }

  /**
   * Creates a shared, empty {@link SortedSet} instance.
   */
  public static <E> Shared<SortedSet<E>> newTreeSet()
  {
    return new Shared<SortedSet<E>>()
    {
      protected SortedSet<E> create()
      {
        return new TreeSet<E>();
      }

      protected SortedSet<E> unmodifiable(SortedSet<E> collection)
      {
        return Collections.unmodifiableSortedSet(collection);
      }

      protected SortedSet<E> copy(SortedSet<E> collection)
      {
        return new TreeSet<E>(collection);
      }
    };
  }

  /**
   * Creates a shared, empty {@link HashMap} instance.
   */
  public static <K, V> Shared<Map<K, V>> newHashMap()
  {
    return new Shared<Map<K, V>>()
    {
      protected Map<K, V> create()
      {
        return new HashMap<K, V>();
      }

      protected Map<K, V> unmodifiable(Map<K, V> collection)
      {
        return Collections.unmodifiableMap(collection);
      }

      protected Map<K, V> copy(Map<K, V> collection)
      {
        return new HashMap<K, V>(collection);
      }
    };
  }

  /**
   * Creates a shared, empty {@link EnumMap} instance.
   */
  public static <K extends Enum<K>, V> Shared<Map<K, V>> newEnumMap(final Class<K> keyType)
  {
    return new Shared<Map<K, V>>()
    {
      protected Map<K, V> create()
      {
        return new EnumMap<K, V>(keyType);
      }

      protected Map<K, V> unmodifiable(Map<K, V> collection)
      {
        return Collections.unmodifiableMap(collection);
      }

      protected Map<K, V> copy(Map<K, V> collection)
      {
        return collection.isEmpty() ? create() : new EnumMap<K, V>(collection);
      }
    };
  }

  //
  // API
  //

  /**
   *
   */
  public final C asMutable()
  {
    assert collection != null;

    if (isImmutable)
    {
      collection = copy(collection);
      isImmutable = false;
    }

    return collection;
  }

  /**
   *
   */
  public void setImmutable(C collection)
  {
    if (collection == null)
      throw new NullPointerException("collection");

    this.collection = collection;
    isImmutable = true;
  }

  /**
   *
   */
  public final C toImmutable()
  {
    assert collection != null;

    if (!isImmutable)
    {
      collection = unmodifiable(collection);
      isImmutable = true;
    }

    return collection;
  }

  /**
   *
   */
  protected abstract C create();

  /**
   *
   */
  protected abstract C unmodifiable(C collection);

  /**
   *
   */
  protected abstract C copy(C collection);
}
