package com.olympya.util.xml;

import java.util.*;
import javax.xml.bind.annotation.*;
import javax.xml.bind.annotation.adapters.*;

import static com.google.common.collect.Maps.*;
import static com.google.common.collect.Lists.*;

/**
 * A extensible {@link Map} adapter compatible with the {@link XmlJavaTypeAdapter} annotation.
 *
 * @param <K> the type of keys maintained by the <em>bound</em> map.
 * @param <V> the type of values maintained by the <em>bound</em> map.
 *
 * @author Paulo Mattos
 */
public abstract class XmlMapAdapter<K, V>
extends XmlAdapter<XmlMapAdapter.MapType, Map<K, V>>
{
  private final boolean immutable;

  /**
   * Constructor designed for subclassing.
   */
  protected XmlMapAdapter(boolean immutable)
  {
    this.immutable = immutable;
  }

  /**
   * {@inheritDoc}
   */
  public MapType marshal(Map<K, V> bound) throws Exception
  {
    return new MapType(bound, this);
  }

  /**
   * {@inheritDoc}
   */
  public Map<K, V> unmarshal(MapType value) throws Exception
  {
    return value.toMap(this);
  }

  //
  // Subclass Methods
  //

  /**
   *
   */
  protected abstract String marshalKey(K key);

  /**
   *
   */
  protected abstract String marshalValue(V value);

  /**
   *
   */
  protected abstract K unmarshalKey(String key);

  /**
   *
   */
  protected abstract V unmarshalValue(String value);

  /**
   * A XML map.
   */
  static final class MapType
  {
    @XmlElement(name="entry")
    final List<EntryType> entries;

    <K, V> MapType(Map<K, V> map, XmlMapAdapter<K, V> adapter)
    {
      entries = newArrayList();

      for (Map.Entry<K, V> entry : map.entrySet())
      {
        String key   = adapter.marshalKey(entry.getKey());
        String value = adapter.marshalValue(entry.getValue());

        entries.add(new EntryType(key, value));
      }
    }

    /**
     * No-arg constructor required by the {@link javax.xml.bind} API.
     */
    MapType()
    {
      entries = null;
    }

    <K, V> Map<K, V> toMap(XmlMapAdapter<K, V> adapter)
    {
      Map<K, V> map = newHashMap();

      for (EntryType entry : entries)
      {
        K key   = adapter.unmarshalKey(entry.key);
        V value = adapter.unmarshalValue(entry.value);

        map.put(key, value);
      }

      return adapter.immutable ? Collections.unmodifiableMap(map) : map;
    }
  }

  /**
   * A XML map entry.
   * <p>
   *
   * (Note: {@link XmlAttribute}/{@link XmlValue} need to reference a Java type that maps to text in XML.)
   */
  private static final class EntryType
  {
    @XmlAttribute
    final String key;

    @XmlValue
    final String value;

    EntryType(String key, String value)
    {
      this.key   = key;
      this.value = value;
    }

    /**
     * No-arg constructor required by the {@link javax.xml.bind} API.
     */
    EntryType()
    {
      this.key   = null;
      this.value = null;
    }
  }
}
